@@ -331,8 +331,8 @@ class BaseEstimator:
                     "using version {2}. This might lead to breaking code or "
                     "invalid results. Use at your own risk. "
                     "For more info please refer to:\n"
-                    "https://scikit-learn.org/stable/modules/model_persistence"
-                    ".html#security-maintainability-limitations".format(
+                    "https://scikit-learn.org/stable/model_persistence.html"
+                    "#security-maintainability-limitations".format(
                         self.__class__.__name__, pickle_version, __version__
                     ),
                     UserWarning,
