@@ -5,7 +5,6 @@ estimator.
 # Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
 # License: BSD 3 clause
 # Copyright: INRIA
-from collections.abc import Sequence
 import warnings
 import operator
 import sys
@@ -13,13 +12,14 @@ import time
 
 import numpy as np
 from scipy import linalg
-from joblib import Parallel, delayed
+from joblib import Parallel
 
 from . import empirical_covariance, EmpiricalCovariance, log_likelihood
 
 from ..exceptions import ConvergenceWarning
-from ..utils.validation import check_random_state
-from ..utils.validation import _deprecate_positional_args
+from ..utils.validation import _is_arraylike_not_scalar, check_random_state
+from ..utils.fixes import delayed
+
 # mypy error: Module 'sklearn.linear_model' has no attribute '_cd_fast'
 from ..linear_model import _cd_fast as cd_fast  # type: ignore
 from ..linear_model import lars_path_gram
@@ -36,9 +36,8 @@ def _objective(mle, precision_, alpha):
     penalisation term to promote sparsity
     """
     p = precision_.shape[0]
-    cost = - 2. * log_likelihood(mle, precision_) + p * np.log(2 * np.pi)
-    cost += alpha * (np.abs(precision_).sum()
-                     - np.abs(np.diag(precision_)).sum())
+    cost = -2.0 * log_likelihood(mle, precision_) + p * np.log(2 * np.pi)
+    cost += alpha * (np.abs(precision_).sum() - np.abs(np.diag(precision_)).sum())
     return cost
 
 
@@ -50,8 +49,7 @@ def _dual_gap(emp_cov, precision_, alpha):
     """
     gap = np.sum(emp_cov * precision_)
     gap -= precision_.shape[0]
-    gap += alpha * (np.abs(precision_).sum()
-                    - np.abs(np.diag(precision_)).sum())
+    gap += alpha * (np.abs(precision_).sum() - np.abs(np.diag(precision_)).sum())
     return gap
 
 
@@ -70,17 +68,49 @@ def alpha_max(emp_cov):
     bound for alpha is given by `max(abs(Xy))`, the result follows.
     """
     A = np.copy(emp_cov)
-    A.flat[::A.shape[0] + 1] = 0
+    A.flat[:: A.shape[0] + 1] = 0
     return np.max(np.abs(A))
 
 
+class _DictWithDeprecatedKeys(dict):
+    """Dictionary with deprecated keys.
+
+    Currently only be used in GraphicalLassoCV to deprecate keys"""
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self._deprecated_key_to_new_key = {}
+
+    def __getitem__(self, key):
+        if key in self._deprecated_key_to_new_key:
+            warnings.warn(
+                f"Key: '{key}', is deprecated in 1.0 and will be "
+                f"removed in 1.2. Use '{self._deprecated_key_to_new_key[key]}' instead",
+                FutureWarning,
+            )
+        return super().__getitem__(key)
+
+    def _set_deprecated(self, value, *, new_key, deprecated_key):
+        self._deprecated_key_to_new_key[deprecated_key] = new_key
+        self[new_key] = self[deprecated_key] = value
+
+
 # The g-lasso algorithm
-@_deprecate_positional_args
-def graphical_lasso(emp_cov, alpha, *, cov_init=None, mode='cd', tol=1e-4,
-                    enet_tol=1e-4, max_iter=100, verbose=False,
-                    return_costs=False, eps=np.finfo(np.float64).eps,
-                    return_n_iter=False):
-    """l1-penalized covariance estimator
+def graphical_lasso(
+    emp_cov,
+    alpha,
+    *,
+    cov_init=None,
+    mode="cd",
+    tol=1e-4,
+    enet_tol=1e-4,
+    max_iter=100,
+    verbose=False,
+    return_costs=False,
+    eps=np.finfo(np.float64).eps,
+    return_n_iter=False,
+):
+    """L1-penalized covariance estimator.
 
     Read more in the :ref:`User Guide <sparse_inverse_covariance>`.
 
@@ -98,7 +128,8 @@ def graphical_lasso(emp_cov, alpha, *, cov_init=None, mode='cd', tol=1e-4,
         Range is (0, inf].
 
     cov_init : array of shape (n_features, n_features), default=None
-        The initial guess for the covariance.
+        The initial guess for the covariance. If None, then the empirical
+        covariance is used.
 
     mode : {'cd', 'lars'}, default='cd'
         The Lasso solver to use: coordinate descent or LARS. Use LARS for
@@ -151,7 +182,10 @@ def graphical_lasso(emp_cov, alpha, *, cov_init=None, mode='cd', tol=1e-4,
 
     See Also
     --------
-    GraphicalLasso, GraphicalLassoCV
+    GraphicalLasso : Sparse inverse covariance estimation
+        with an l1-penalized estimator.
+    GraphicalLassoCV : Sparse inverse covariance with
+        cross-validated choice of the l1 penalty.
 
     Notes
     -----
@@ -166,7 +200,7 @@ def graphical_lasso(emp_cov, alpha, *, cov_init=None, mode='cd', tol=1e-4,
     if alpha == 0:
         if return_costs:
             precision_ = linalg.inv(emp_cov)
-            cost = - 2. * log_likelihood(emp_cov, precision_)
+            cost = -2.0 * log_likelihood(emp_cov, precision_)
             cost += n_features * np.log(2 * np.pi)
             d_gap = np.sum(emp_cov * precision_) - n_features
             if return_n_iter:
@@ -189,23 +223,23 @@ def graphical_lasso(emp_cov, alpha, *, cov_init=None, mode='cd', tol=1e-4,
     # conservative stand-point on the initial conditions, and it tends to
     # make the convergence go faster.
     covariance_ *= 0.95
-    diagonal = emp_cov.flat[::n_features + 1]
-    covariance_.flat[::n_features + 1] = diagonal
+    diagonal = emp_cov.flat[:: n_features + 1]
+    covariance_.flat[:: n_features + 1] = diagonal
     precision_ = linalg.pinvh(covariance_)
 
     indices = np.arange(n_features)
     costs = list()
     # The different l1 regression solver have different numerical errors
-    if mode == 'cd':
-        errors = dict(over='raise', invalid='ignore')
+    if mode == "cd":
+        errors = dict(over="raise", invalid="ignore")
     else:
-        errors = dict(invalid='raise')
+        errors = dict(invalid="raise")
     try:
         # be robust to the max_iter=0 edge case, see:
         # https://github.com/scikit-learn/scikit-learn/issues/4134
         d_gap = np.inf
         # set a sub_covariance buffer
-        sub_covariance = np.copy(covariance_[1:, 1:], order='C')
+        sub_covariance = np.copy(covariance_[1:, 1:], order="C")
         for i in range(max_iter):
             for idx in range(n_features):
                 # To keep the contiguous matrix `sub_covariance` equal to
@@ -219,54 +253,73 @@ def graphical_lasso(emp_cov, alpha, *, cov_init=None, mode='cd', tol=1e-4,
                     sub_covariance[:] = covariance_[1:, 1:]
                 row = emp_cov[idx, indices != idx]
                 with np.errstate(**errors):
-                    if mode == 'cd':
+                    if mode == "cd":
                         # Use coordinate descent
-                        coefs = -(precision_[indices != idx, idx]
-                                  / (precision_[idx, idx] + 1000 * eps))
+                        coefs = -(
+                            precision_[indices != idx, idx]
+                            / (precision_[idx, idx] + 1000 * eps)
+                        )
                         coefs, _, _, _ = cd_fast.enet_coordinate_descent_gram(
-                            coefs, alpha, 0, sub_covariance,
-                            row, row, max_iter, enet_tol,
-                            check_random_state(None), False)
+                            coefs,
+                            alpha,
+                            0,
+                            sub_covariance,
+                            row,
+                            row,
+                            max_iter,
+                            enet_tol,
+                            check_random_state(None),
+                            False,
+                        )
                     else:
                         # Use LARS
                         _, _, coefs = lars_path_gram(
-                            Xy=row, Gram=sub_covariance, n_samples=row.size,
-                            alpha_min=alpha / (n_features - 1), copy_Gram=True,
-                            eps=eps, method='lars', return_path=False)
+                            Xy=row,
+                            Gram=sub_covariance,
+                            n_samples=row.size,
+                            alpha_min=alpha / (n_features - 1),
+                            copy_Gram=True,
+                            eps=eps,
+                            method="lars",
+                            return_path=False,
+                        )
                 # Update the precision matrix
-                precision_[idx, idx] = (
-                    1. / (covariance_[idx, idx]
-                          - np.dot(covariance_[indices != idx, idx], coefs)))
-                precision_[indices != idx, idx] = (- precision_[idx, idx]
-                                                   * coefs)
-                precision_[idx, indices != idx] = (- precision_[idx, idx]
-                                                   * coefs)
+                precision_[idx, idx] = 1.0 / (
+                    covariance_[idx, idx]
+                    - np.dot(covariance_[indices != idx, idx], coefs)
+                )
+                precision_[indices != idx, idx] = -precision_[idx, idx] * coefs
+                precision_[idx, indices != idx] = -precision_[idx, idx] * coefs
                 coefs = np.dot(sub_covariance, coefs)
                 covariance_[idx, indices != idx] = coefs
                 covariance_[indices != idx, idx] = coefs
             if not np.isfinite(precision_.sum()):
-                raise FloatingPointError('The system is too ill-conditioned '
-                                         'for this solver')
+                raise FloatingPointError(
+                    "The system is too ill-conditioned for this solver"
+                )
             d_gap = _dual_gap(emp_cov, precision_, alpha)
             cost = _objective(emp_cov, precision_, alpha)
             if verbose:
-                print('[graphical_lasso] Iteration '
-                      '% 3i, cost % 3.2e, dual gap %.3e'
-                      % (i, cost, d_gap))
+                print(
+                    "[graphical_lasso] Iteration % 3i, cost % 3.2e, dual gap %.3e"
+                    % (i, cost, d_gap)
+                )
             if return_costs:
                 costs.append((cost, d_gap))
             if np.abs(d_gap) < tol:
                 break
             if not np.isfinite(cost) and i > 0:
-                raise FloatingPointError('Non SPD result: the system is '
-                                         'too ill-conditioned for this solver')
+                raise FloatingPointError(
+                    "Non SPD result: the system is too ill-conditioned for this solver"
+                )
         else:
-            warnings.warn('graphical_lasso: did not converge after '
-                          '%i iteration: dual gap: %.3e'
-                          % (max_iter, d_gap), ConvergenceWarning)
+            warnings.warn(
+                "graphical_lasso: did not converge after %i iteration: dual gap: %.3e"
+                % (max_iter, d_gap),
+                ConvergenceWarning,
+            )
     except FloatingPointError as e:
-        e.args = (e.args[0]
-                  + '. The system is too ill-conditioned for this solver',)
+        e.args = (e.args[0] + ". The system is too ill-conditioned for this solver",)
         raise e
 
     if return_costs:
@@ -338,6 +391,23 @@ class GraphicalLasso(EmpiricalCovariance):
     n_iter_ : int
         Number of iterations run.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
+    See Also
+    --------
+    graphical_lasso : L1-penalized covariance estimator.
+    GraphicalLassoCV : Sparse inverse covariance with
+        cross-validated choice of the l1 penalty.
+
     Examples
     --------
     >>> import numpy as np
@@ -358,14 +428,19 @@ class GraphicalLasso(EmpiricalCovariance):
            [0.019, 0.034, 0.093, 0.69 ]])
     >>> np.around(cov.location_, decimals=3)
     array([0.073, 0.04 , 0.038, 0.143])
-
-    See Also
-    --------
-    graphical_lasso, GraphicalLassoCV
     """
-    @_deprecate_positional_args
-    def __init__(self, alpha=.01, *, mode='cd', tol=1e-4, enet_tol=1e-4,
-                 max_iter=100, verbose=False, assume_centered=False):
+
+    def __init__(
+        self,
+        alpha=0.01,
+        *,
+        mode="cd",
+        tol=1e-4,
+        enet_tol=1e-4,
+        max_iter=100,
+        verbose=False,
+        assume_centered=False,
+    ):
         super().__init__(assume_centered=assume_centered)
         self.alpha = alpha
         self.mode = mode
@@ -375,40 +450,54 @@ class GraphicalLasso(EmpiricalCovariance):
         self.verbose = verbose
 
     def fit(self, X, y=None):
-        """Fits the GraphicalLasso model to X.
+        """Fit the GraphicalLasso model to X.
 
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
-            Data from which to compute the covariance estimate
+            Data from which to compute the covariance estimate.
 
         y : Ignored
-            Not used, present for API consistence purpose.
+            Not used, present for API consistency by convention.
 
         Returns
         -------
         self : object
+            Returns the instance itself.
         """
         # Covariance does not make sense for a single feature
-        X = self._validate_data(X, ensure_min_features=2, ensure_min_samples=2,
-                                estimator=self)
+        X = self._validate_data(X, ensure_min_features=2, ensure_min_samples=2)
 
         if self.assume_centered:
             self.location_ = np.zeros(X.shape[1])
         else:
             self.location_ = X.mean(0)
-        emp_cov = empirical_covariance(
-            X, assume_centered=self.assume_centered)
+        emp_cov = empirical_covariance(X, assume_centered=self.assume_centered)
         self.covariance_, self.precision_, self.n_iter_ = graphical_lasso(
-            emp_cov, alpha=self.alpha, mode=self.mode, tol=self.tol,
-            enet_tol=self.enet_tol, max_iter=self.max_iter,
-            verbose=self.verbose, return_n_iter=True)
+            emp_cov,
+            alpha=self.alpha,
+            mode=self.mode,
+            tol=self.tol,
+            enet_tol=self.enet_tol,
+            max_iter=self.max_iter,
+            verbose=self.verbose,
+            return_n_iter=True,
+        )
         return self
 
 
 # Cross-validation with GraphicalLasso
-def graphical_lasso_path(X, alphas, cov_init=None, X_test=None, mode='cd',
-                         tol=1e-4, enet_tol=1e-4, max_iter=100, verbose=False):
+def graphical_lasso_path(
+    X,
+    alphas,
+    cov_init=None,
+    X_test=None,
+    mode="cd",
+    tol=1e-4,
+    enet_tol=1e-4,
+    max_iter=100,
+    verbose=False,
+):
     """l1-penalized covariance estimator along a path of decreasing alphas
 
     Read more in the :ref:`User Guide <sparse_inverse_covariance>`.
@@ -481,8 +570,15 @@ def graphical_lasso_path(X, alphas, cov_init=None, X_test=None, mode='cd',
         try:
             # Capture the errors, and move on
             covariance_, precision_ = graphical_lasso(
-                emp_cov, alpha=alpha, cov_init=covariance_, mode=mode, tol=tol,
-                enet_tol=enet_tol, max_iter=max_iter, verbose=inner_verbose)
+                emp_cov,
+                alpha=alpha,
+                cov_init=covariance_,
+                mode=mode,
+                tol=tol,
+                enet_tol=enet_tol,
+                max_iter=max_iter,
+                verbose=inner_verbose,
+            )
             covariances_.append(covariance_)
             precisions_.append(precision_)
             if X_test is not None:
@@ -496,13 +592,15 @@ def graphical_lasso_path(X, alphas, cov_init=None, X_test=None, mode='cd',
                 this_score = -np.inf
             scores_.append(this_score)
         if verbose == 1:
-            sys.stderr.write('.')
+            sys.stderr.write(".")
         elif verbose > 1:
             if X_test is not None:
-                print('[graphical_lasso_path] alpha: %.2e, score: %.2e'
-                      % (alpha, this_score))
+                print(
+                    "[graphical_lasso_path] alpha: %.2e, score: %.2e"
+                    % (alpha, this_score)
+                )
             else:
-                print('[graphical_lasso_path] alpha: %.2e' % alpha)
+                print("[graphical_lasso_path] alpha: %.2e" % alpha)
     if X_test is not None:
         return covariances_, precisions_, scores_
     return covariances_, precisions_
@@ -567,7 +665,7 @@ class GraphicalLassoCV(GraphicalLasso):
         stable.
 
     n_jobs : int, default=None
-        number of jobs to run in parallel.
+        Number of jobs to run in parallel.
         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
         for more details.
@@ -599,15 +697,85 @@ class GraphicalLassoCV(GraphicalLasso):
     alpha_ : float
         Penalization parameter selected.
 
-    cv_alphas_ : list of shape (n_alphas,), dtype=float
-        All penalization parameters explored.
+    cv_results_ : dict of ndarrays
+        A dict with keys:
 
-    grid_scores_ : ndarray of shape (n_alphas, n_folds)
-        Log-likelihood score on left-out data across folds.
+        alphas : ndarray of shape (n_alphas,)
+            All penalization parameters explored.
+
+        split(k)_test_score : ndarray of shape (n_alphas,)
+            Log-likelihood score on left-out data across (k)th fold.
+
+            .. versionadded:: 1.0
+
+        mean_test_score : ndarray of shape (n_alphas,)
+            Mean of scores over the folds.
+
+            .. versionadded:: 1.0
+
+        std_test_score : ndarray of shape (n_alphas,)
+            Standard deviation of scores over the folds.
+
+            .. versionadded:: 1.0
+
+        split(k)_score : ndarray of shape (n_alphas,)
+            Log-likelihood score on left-out data across (k)th fold.
+
+            .. deprecated:: 1.0
+                `split(k)_score` is deprecated in 1.0 and will be removed in 1.2.
+                Use `split(k)_test_score` instead.
+
+        mean_score : ndarray of shape (n_alphas,)
+            Mean of scores over the folds.
+
+            .. deprecated:: 1.0
+                `mean_score` is deprecated in 1.0 and will be removed in 1.2.
+                Use `mean_test_score` instead.
+
+        std_score : ndarray of shape (n_alphas,)
+            Standard deviation of scores over the folds.
+
+            .. deprecated:: 1.0
+                `std_score` is deprecated in 1.0 and will be removed in 1.2.
+                Use `std_test_score` instead.
+
+        .. versionadded:: 0.24
 
     n_iter_ : int
         Number of iterations run for the optimal alpha.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
+    See Also
+    --------
+    graphical_lasso : L1-penalized covariance estimator.
+    GraphicalLasso : Sparse inverse covariance estimation
+        with an l1-penalized estimator.
+
+    Notes
+    -----
+    The search for the optimal penalization parameter (`alpha`) is done on an
+    iteratively refined grid: first the cross-validated scores on a grid are
+    computed, then a new refined grid is centered around the maximum, and so
+    on.
+
+    One of the challenges which is faced here is that the solvers can
+    fail to converge to a well-conditioned estimate. The corresponding
+    values of `alpha` then come out as missing values, but the optimum may
+    be close to these missing values.
+
+    In `fit`, once the best parameter `alpha` is found through
+    cross-validation, the model is fit again using the entire training set.
+
     Examples
     --------
     >>> import numpy as np
@@ -628,58 +796,58 @@ class GraphicalLassoCV(GraphicalLasso):
            [0.017, 0.036, 0.094, 0.69 ]])
     >>> np.around(cov.location_, decimals=3)
     array([0.073, 0.04 , 0.038, 0.143])
-
-    See Also
-    --------
-    graphical_lasso, GraphicalLasso
-
-    Notes
-    -----
-    The search for the optimal penalization parameter (alpha) is done on an
-    iteratively refined grid: first the cross-validated scores on a grid are
-    computed, then a new refined grid is centered around the maximum, and so
-    on.
-
-    One of the challenges which is faced here is that the solvers can
-    fail to converge to a well-conditioned estimate. The corresponding
-    values of alpha then come out as missing values, but the optimum may
-    be close to these missing values.
     """
-    @_deprecate_positional_args
-    def __init__(self, *, alphas=4, n_refinements=4, cv=None, tol=1e-4,
-                 enet_tol=1e-4, max_iter=100, mode='cd', n_jobs=None,
-                 verbose=False, assume_centered=False):
+
+    def __init__(
+        self,
+        *,
+        alphas=4,
+        n_refinements=4,
+        cv=None,
+        tol=1e-4,
+        enet_tol=1e-4,
+        max_iter=100,
+        mode="cd",
+        n_jobs=None,
+        verbose=False,
+        assume_centered=False,
+    ):
         super().__init__(
-            mode=mode, tol=tol, verbose=verbose, enet_tol=enet_tol,
-            max_iter=max_iter, assume_centered=assume_centered)
+            mode=mode,
+            tol=tol,
+            verbose=verbose,
+            enet_tol=enet_tol,
+            max_iter=max_iter,
+            assume_centered=assume_centered,
+        )
         self.alphas = alphas
         self.n_refinements = n_refinements
         self.cv = cv
         self.n_jobs = n_jobs
 
     def fit(self, X, y=None):
-        """Fits the GraphicalLasso covariance model to X.
+        """Fit the GraphicalLasso covariance model to X.
 
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
-            Data from which to compute the covariance estimate
+            Data from which to compute the covariance estimate.
 
         y : Ignored
-            Not used, present for API consistence purpose.
+            Not used, present for API consistency by convention.
 
         Returns
         -------
         self : object
+            Returns the instance itself.
         """
         # Covariance does not make sense for a single feature
-        X = self._validate_data(X, ensure_min_features=2, estimator=self)
+        X = self._validate_data(X, ensure_min_features=2)
         if self.assume_centered:
             self.location_ = np.zeros(X.shape[1])
         else:
             self.location_ = X.mean(0)
-        emp_cov = empirical_covariance(
-            X, assume_centered=self.assume_centered)
+        emp_cov = empirical_covariance(X, assume_centered=self.assume_centered)
 
         cv = check_cv(self.cv, y, classifier=False)
 
@@ -688,15 +856,14 @@ class GraphicalLassoCV(GraphicalLasso):
         n_alphas = self.alphas
         inner_verbose = max(0, self.verbose - 1)
 
-        if isinstance(n_alphas, Sequence):
+        if _is_arraylike_not_scalar(n_alphas):
             alphas = self.alphas
             n_refinements = 1
         else:
             n_refinements = self.n_refinements
             alpha_1 = alpha_max(emp_cov)
             alpha_0 = 1e-2 * alpha_1
-            alphas = np.logspace(np.log10(alpha_0), np.log10(alpha_1),
-                                 n_alphas)[::-1]
+            alphas = np.logspace(np.log10(alpha_0), np.log10(alpha_1), n_alphas)[::-1]
 
         t0 = time.time()
         for i in range(n_refinements):
@@ -704,23 +871,25 @@ class GraphicalLassoCV(GraphicalLasso):
                 # No need to see the convergence warnings on this grid:
                 # they will always be points that will not converge
                 # during the cross-validation
-                warnings.simplefilter('ignore', ConvergenceWarning)
+                warnings.simplefilter("ignore", ConvergenceWarning)
                 # Compute the cross-validated loss on the current grid
 
                 # NOTE: Warm-restarting graphical_lasso_path has been tried,
                 # and this did not allow to gain anything
                 # (same execution time with or without).
-                this_path = Parallel(
-                    n_jobs=self.n_jobs,
-                    verbose=self.verbose
-                )(delayed(graphical_lasso_path)(X[train], alphas=alphas,
-                                                X_test=X[test], mode=self.mode,
-                                                tol=self.tol,
-                                                enet_tol=self.enet_tol,
-                                                max_iter=int(.1 *
-                                                             self.max_iter),
-                                                verbose=inner_verbose)
-                  for train, test in cv.split(X, y))
+                this_path = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(
+                    delayed(graphical_lasso_path)(
+                        X[train],
+                        alphas=alphas,
+                        X_test=X[test],
+                        mode=self.mode,
+                        tol=self.tol,
+                        enet_tol=self.enet_tol,
+                        max_iter=int(0.1 * self.max_iter),
+                        verbose=inner_verbose,
+                    )
+                    for train, test in cv.split(X, y)
+                )
 
             # Little danse to transform the list in what we need
             covs, _, scores = zip(*this_path)
@@ -736,7 +905,7 @@ class GraphicalLassoCV(GraphicalLasso):
             last_finite_idx = 0
             for index, (alpha, scores, _) in enumerate(path):
                 this_score = np.mean(scores)
-                if this_score >= .1 / np.finfo(np.float64).eps:
+                if this_score >= 0.1 / np.finfo(np.float64).eps:
                     this_score = np.nan
                 if np.isfinite(this_score):
                     last_finite_idx = index
@@ -751,8 +920,7 @@ class GraphicalLassoCV(GraphicalLasso):
                 # non-zero coefficients
                 alpha_1 = path[0][0]
                 alpha_0 = path[1][0]
-            elif (best_index == last_finite_idx
-                    and not best_index == len(path) - 1):
+            elif best_index == last_finite_idx and not best_index == len(path) - 1:
                 # We have non-converged models on the upper bound of the
                 # grid, we need to refine the grid there
                 alpha_1 = path[best_index][0]
@@ -764,31 +932,65 @@ class GraphicalLassoCV(GraphicalLasso):
                 alpha_1 = path[best_index - 1][0]
                 alpha_0 = path[best_index + 1][0]
 
-            if not isinstance(n_alphas, Sequence):
-                alphas = np.logspace(np.log10(alpha_1), np.log10(alpha_0),
-                                     n_alphas + 2)
+            if not _is_arraylike_not_scalar(n_alphas):
+                alphas = np.logspace(np.log10(alpha_1), np.log10(alpha_0), n_alphas + 2)
                 alphas = alphas[1:-1]
 
             if self.verbose and n_refinements > 1:
-                print('[GraphicalLassoCV] Done refinement % 2i out of'
-                      ' %i: % 3is' % (i + 1, n_refinements, time.time() - t0))
+                print(
+                    "[GraphicalLassoCV] Done refinement % 2i out of %i: % 3is"
+                    % (i + 1, n_refinements, time.time() - t0)
+                )
 
         path = list(zip(*path))
         grid_scores = list(path[1])
         alphas = list(path[0])
         # Finally, compute the score with alpha = 0
         alphas.append(0)
-        grid_scores.append(cross_val_score(EmpiricalCovariance(), X,
-                                           cv=cv, n_jobs=self.n_jobs,
-                                           verbose=inner_verbose))
-        self.grid_scores_ = np.array(grid_scores)
+        grid_scores.append(
+            cross_val_score(
+                EmpiricalCovariance(),
+                X,
+                cv=cv,
+                n_jobs=self.n_jobs,
+                verbose=inner_verbose,
+            )
+        )
+        grid_scores = np.array(grid_scores)
+
+        # TODO(1.2): Use normal dict for cv_results_ instead of _DictWithDeprecatedKeys
+        self.cv_results_ = _DictWithDeprecatedKeys(alphas=np.array(alphas))
+
+        for i in range(grid_scores.shape[1]):
+            self.cv_results_._set_deprecated(
+                grid_scores[:, i],
+                new_key=f"split{i}_test_score",
+                deprecated_key=f"split{i}_score",
+            )
+
+        self.cv_results_._set_deprecated(
+            np.mean(grid_scores, axis=1),
+            new_key="mean_test_score",
+            deprecated_key="mean_score",
+        )
+        self.cv_results_._set_deprecated(
+            np.std(grid_scores, axis=1),
+            new_key="std_test_score",
+            deprecated_key="std_score",
+        )
+
         best_alpha = alphas[best_index]
         self.alpha_ = best_alpha
-        self.cv_alphas_ = alphas
 
         # Finally fit the model with the selected alpha
         self.covariance_, self.precision_, self.n_iter_ = graphical_lasso(
-            emp_cov, alpha=best_alpha, mode=self.mode, tol=self.tol,
-            enet_tol=self.enet_tol, max_iter=self.max_iter,
-            verbose=inner_verbose, return_n_iter=True)
+            emp_cov,
+            alpha=best_alpha,
+            mode=self.mode,
+            tol=self.tol,
+            enet_tol=self.enet_tol,
+            max_iter=self.max_iter,
+            verbose=inner_verbose,
+            return_n_iter=True,
+        )
         return self
