@@ -20,12 +20,11 @@ from scipy.sparse import issparse
 from scipy.sparse.linalg import svds
 
 from ._base import _BasePCA
-from ..utils import check_random_state
+from ..utils import check_random_state, check_scalar
 from ..utils._arpack import _init_arpack_v0
 from ..utils.extmath import fast_logdet, randomized_svd, svd_flip
 from ..utils.extmath import stable_cumsum
 from ..utils.validation import check_is_fitted
-from ..utils.validation import _deprecate_positional_args
 
 
 def _assess_dimension(spectrum, rank, n_samples):
@@ -72,29 +71,32 @@ def _assess_dimension(spectrum, rank, n_samples):
         # spectrum[j]) because this will take the log of something very small.
         return -np.inf
 
-    pu = -rank * log(2.)
+    pu = -rank * log(2.0)
     for i in range(1, rank + 1):
-        pu += (gammaln((n_features - i + 1) / 2.) -
-               log(np.pi) * (n_features - i + 1) / 2.)
+        pu += (
+            gammaln((n_features - i + 1) / 2.0)
+            - log(np.pi) * (n_features - i + 1) / 2.0
+        )
 
     pl = np.sum(np.log(spectrum[:rank]))
-    pl = -pl * n_samples / 2.
+    pl = -pl * n_samples / 2.0
 
     v = max(eps, np.sum(spectrum[rank:]) / (n_features - rank))
-    pv = -np.log(v) * n_samples * (n_features - rank) / 2.
+    pv = -np.log(v) * n_samples * (n_features - rank) / 2.0
 
-    m = n_features * rank - rank * (rank + 1.) / 2.
-    pp = log(2. * np.pi) * (m + rank) / 2.
+    m = n_features * rank - rank * (rank + 1.0) / 2.0
+    pp = log(2.0 * np.pi) * (m + rank) / 2.0
 
-    pa = 0.
+    pa = 0.0
     spectrum_ = spectrum.copy()
     spectrum_[rank:n_features] = v
     for i in range(rank):
         for j in range(i + 1, len(spectrum)):
-            pa += log((spectrum[i] - spectrum[j]) *
-                      (1. / spectrum_[j] - 1. / spectrum_[i])) + log(n_samples)
+            pa += log(
+                (spectrum[i] - spectrum[j]) * (1.0 / spectrum_[j] - 1.0 / spectrum_[i])
+            ) + log(n_samples)
 
-    ll = pu + pl + pv + pp - pa / 2. - rank * log(n_samples) / 2.
+    ll = pu + pl + pv + pp - pa / 2.0 - rank * log(n_samples) / 2.0
 
     return ll
 
@@ -201,6 +203,21 @@ class PCA(_BasePCA):
 
         .. versionadded:: 0.18.0
 
+    n_oversamples : int, default=10
+        This parameter is only relevant when `svd_solver="randomized"`.
+        It corresponds to the additional number of random vectors to sample the
+        range of `X` so as to ensure proper conditioning. See
+        :func:`~sklearn.utils.extmath.randomized_svd` for more details.
+
+        .. versionadded:: 1.1
+
+    power_iteration_normalizer : {‘auto’, ‘QR’, ‘LU’, ‘none’}, default=’auto’
+        Power iteration normalizer for randomized SVD solver.
+        Not used by ARPACK. See :func:`~sklearn.utils.extmath.randomized_svd`
+        for more details.
+
+        .. versionadded:: 1.1
+
     random_state : int, RandomState instance or None, default=None
         Used when the 'arpack' or 'randomized' solvers are used. Pass an int
         for reproducible results across multiple function calls.
@@ -212,8 +229,9 @@ class PCA(_BasePCA):
     ----------
     components_ : ndarray of shape (n_components, n_features)
         Principal axes in feature space, representing the directions of
-        maximum variance in the data. The components are sorted by
-        ``explained_variance_``.
+        maximum variance in the data. Equivalently, the right singular
+        vectors of the centered input data, parallel to its eigenvectors.
+        The components are sorted by ``explained_variance_``.
 
     explained_variance_ : ndarray of shape (n_components,)
         The amount of variance explained by each of the selected components.
@@ -265,6 +283,17 @@ class PCA(_BasePCA):
         Equal to the average of (min(n_features, n_samples) - n_components)
         smallest eigenvalues of the covariance matrix of X.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
     See Also
     --------
     KernelPCA : Kernel Principal Component Analysis.
@@ -288,16 +317,16 @@ class PCA(_BasePCA):
     For svd_solver == 'arpack', refer to `scipy.sparse.linalg.svds`.
 
     For svd_solver == 'randomized', see:
-    `Halko, N., Martinsson, P. G., and Tropp, J. A. (2011).
+    :doi:`Halko, N., Martinsson, P. G., and Tropp, J. A. (2011).
     "Finding structure with randomness: Probabilistic algorithms for
     constructing approximate matrix decompositions".
     SIAM review, 53(2), 217-288.
-    <https://doi.org/10.1137/090771806>`_
+    <10.1137/090771806>`
     and also
-    `Martinsson, P. G., Rokhlin, V., and Tygert, M. (2011).
+    :doi:`Martinsson, P. G., Rokhlin, V., and Tygert, M. (2011).
     "A randomized algorithm for the decomposition of matrices".
-    Applied and Computational Harmonic Analysis, 30(1), 47-68
-    <https://doi.org/10.1016/j.acha.2010.02.003>`_.
+    Applied and Computational Harmonic Analysis, 30(1), 47-68.
+    <10.1016/j.acha.2010.02.003>`
 
     Examples
     --------
@@ -328,16 +357,28 @@ class PCA(_BasePCA):
     >>> print(pca.singular_values_)
     [6.30061...]
     """
-    @_deprecate_positional_args
-    def __init__(self, n_components=None, *, copy=True, whiten=False,
-                 svd_solver='auto', tol=0.0, iterated_power='auto',
-                 random_state=None):
+
+    def __init__(
+        self,
+        n_components=None,
+        *,
+        copy=True,
+        whiten=False,
+        svd_solver="auto",
+        tol=0.0,
+        iterated_power="auto",
+        n_oversamples=10,
+        power_iteration_normalizer="auto",
+        random_state=None,
+    ):
         self.n_components = n_components
         self.copy = copy
         self.whiten = whiten
         self.svd_solver = svd_solver
         self.tol = tol
         self.iterated_power = iterated_power
+        self.n_oversamples = n_oversamples
+        self.power_iteration_normalizer = power_iteration_normalizer
         self.random_state = random_state
 
     def fit(self, X, y=None):
@@ -346,16 +387,24 @@ class PCA(_BasePCA):
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
-            Training data, where n_samples is the number of samples
-            and n_features is the number of features.
+            Training data, where `n_samples` is the number of samples
+            and `n_features` is the number of features.
 
         y : Ignored
+            Ignored.
 
         Returns
         -------
         self : object
             Returns the instance itself.
         """
+        check_scalar(
+            self.n_oversamples,
+            "n_oversamples",
+            min_val=1,
+            target_type=numbers.Integral,
+        )
+
         self._fit(X)
         return self
 
@@ -365,10 +414,11 @@ class PCA(_BasePCA):
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
-            Training data, where n_samples is the number of samples
-            and n_features is the number of features.
+            Training data, where `n_samples` is the number of samples
+            and `n_features` is the number of features.
 
         y : Ignored
+            Ignored.
 
         Returns
         -------
@@ -381,14 +431,14 @@ class PCA(_BasePCA):
         C-ordered array, use 'np.ascontiguousarray'.
         """
         U, S, Vt = self._fit(X)
-        U = U[:, :self.n_components_]
+        U = U[:, : self.n_components_]
 
         if self.whiten:
             # X_new = X * V / S * sqrt(n_samples) = U * sqrt(n_samples)
             U *= sqrt(X.shape[0] - 1)
         else:
             # X_new = X * V = U * S * Vt * V = U * S
-            U *= S[:self.n_components_]
+            U *= S[: self.n_components_]
 
         return U
 
@@ -398,15 +448,18 @@ class PCA(_BasePCA):
         # Raise an error for sparse input.
         # This is more informative than the generic one raised by check_array.
         if issparse(X):
-            raise TypeError('PCA does not support sparse input. See '
-                            'TruncatedSVD for a possible alternative.')
+            raise TypeError(
+                "PCA does not support sparse input. See "
+                "TruncatedSVD for a possible alternative."
+            )
 
-        X = self._validate_data(X, dtype=[np.float64, np.float32],
-                                ensure_2d=True, copy=self.copy)
+        X = self._validate_data(
+            X, dtype=[np.float64, np.float32], ensure_2d=True, copy=self.copy
+        )
 
         # Handle n_components==None
         if self.n_components is None:
-            if self.svd_solver != 'arpack':
+            if self.svd_solver != "arpack":
                 n_components = min(X.shape)
             else:
                 n_components = min(X.shape) - 1
@@ -415,44 +468,48 @@ class PCA(_BasePCA):
 
         # Handle svd_solver
         self._fit_svd_solver = self.svd_solver
-        if self._fit_svd_solver == 'auto':
+        if self._fit_svd_solver == "auto":
             # Small problem or n_components == 'mle', just call full PCA
-            if max(X.shape) <= 500 or n_components == 'mle':
-                self._fit_svd_solver = 'full'
-            elif n_components >= 1 and n_components < .8 * min(X.shape):
-                self._fit_svd_solver = 'randomized'
+            if max(X.shape) <= 500 or n_components == "mle":
+                self._fit_svd_solver = "full"
+            elif n_components >= 1 and n_components < 0.8 * min(X.shape):
+                self._fit_svd_solver = "randomized"
             # This is also the case of n_components in (0,1)
             else:
-                self._fit_svd_solver = 'full'
+                self._fit_svd_solver = "full"
 
         # Call different fits for either full or truncated SVD
-        if self._fit_svd_solver == 'full':
+        if self._fit_svd_solver == "full":
             return self._fit_full(X, n_components)
-        elif self._fit_svd_solver in ['arpack', 'randomized']:
+        elif self._fit_svd_solver in ["arpack", "randomized"]:
             return self._fit_truncated(X, n_components, self._fit_svd_solver)
         else:
-            raise ValueError("Unrecognized svd_solver='{0}'"
-                             "".format(self._fit_svd_solver))
+            raise ValueError(
+                "Unrecognized svd_solver='{0}'".format(self._fit_svd_solver)
+            )
 
     def _fit_full(self, X, n_components):
         """Fit the model by computing full SVD on X."""
         n_samples, n_features = X.shape
 
-        if n_components == 'mle':
+        if n_components == "mle":
             if n_samples < n_features:
-                raise ValueError("n_components='mle' is only supported "
-                                 "if n_samples >= n_features")
+                raise ValueError(
+                    "n_components='mle' is only supported if n_samples >= n_features"
+                )
         elif not 0 <= n_components <= min(n_samples, n_features):
-            raise ValueError("n_components=%r must be between 0 and "
-                             "min(n_samples, n_features)=%r with "
-                             "svd_solver='full'"
-                             % (n_components, min(n_samples, n_features)))
+            raise ValueError(
+                "n_components=%r must be between 0 and "
+                "min(n_samples, n_features)=%r with "
+                "svd_solver='full'" % (n_components, min(n_samples, n_features))
+            )
         elif n_components >= 1:
             if not isinstance(n_components, numbers.Integral):
-                raise ValueError("n_components=%r must be of type int "
-                                 "when greater than or equal to 1, "
-                                 "was of type=%r"
-                                 % (n_components, type(n_components)))
+                raise ValueError(
+                    "n_components=%r must be of type int "
+                    "when greater than or equal to 1, "
+                    "was of type=%r" % (n_components, type(n_components))
+                )
 
         # Center data
         self.mean_ = np.mean(X, axis=0)
@@ -465,15 +522,14 @@ class PCA(_BasePCA):
         components_ = Vt
 
         # Get variance explained by singular values
-        explained_variance_ = (S ** 2) / (n_samples - 1)
+        explained_variance_ = (S**2) / (n_samples - 1)
         total_var = explained_variance_.sum()
         explained_variance_ratio_ = explained_variance_ / total_var
         singular_values_ = S.copy()  # Store the singular values.
 
         # Postprocess the number of components required
-        if n_components == 'mle':
-            n_components = \
-                _infer_dimension(explained_variance_, n_samples)
+        if n_components == "mle":
+            n_components = _infer_dimension(explained_variance_, n_samples)
         elif 0 < n_components < 1.0:
             # number of components for which the cumulated explained
             # variance percentage is superior to the desired threshold
@@ -481,21 +537,19 @@ class PCA(_BasePCA):
             # their variance is always greater than n_components float
             # passed. More discussion in issue: #15669
             ratio_cumsum = stable_cumsum(explained_variance_ratio_)
-            n_components = np.searchsorted(ratio_cumsum, n_components,
-                                           side='right') + 1
+            n_components = np.searchsorted(ratio_cumsum, n_components, side="right") + 1
         # Compute noise covariance using Probabilistic PCA model
         # The sigma2 maximum likelihood (cf. eq. 12.46)
         if n_components < min(n_features, n_samples):
             self.noise_variance_ = explained_variance_[n_components:].mean()
         else:
-            self.noise_variance_ = 0.
+            self.noise_variance_ = 0.0
 
         self.n_samples_, self.n_features_ = n_samples, n_features
         self.components_ = components_[:n_components]
         self.n_components_ = n_components
         self.explained_variance_ = explained_variance_[:n_components]
-        self.explained_variance_ratio_ = \
-            explained_variance_ratio_[:n_components]
+        self.explained_variance_ratio_ = explained_variance_ratio_[:n_components]
         self.singular_values_ = singular_values_[:n_components]
 
         return U, S, Vt
@@ -507,26 +561,30 @@ class PCA(_BasePCA):
         n_samples, n_features = X.shape
 
         if isinstance(n_components, str):
-            raise ValueError("n_components=%r cannot be a string "
-                             "with svd_solver='%s'"
-                             % (n_components, svd_solver))
+            raise ValueError(
+                "n_components=%r cannot be a string with svd_solver='%s'"
+                % (n_components, svd_solver)
+            )
         elif not 1 <= n_components <= min(n_samples, n_features):
-            raise ValueError("n_components=%r must be between 1 and "
-                             "min(n_samples, n_features)=%r with "
-                             "svd_solver='%s'"
-                             % (n_components, min(n_samples, n_features),
-                                svd_solver))
+            raise ValueError(
+                "n_components=%r must be between 1 and "
+                "min(n_samples, n_features)=%r with "
+                "svd_solver='%s'"
+                % (n_components, min(n_samples, n_features), svd_solver)
+            )
         elif not isinstance(n_components, numbers.Integral):
-            raise ValueError("n_components=%r must be of type int "
-                             "when greater than or equal to 1, was of type=%r"
-                             % (n_components, type(n_components)))
-        elif svd_solver == 'arpack' and n_components == min(n_samples,
-                                                            n_features):
-            raise ValueError("n_components=%r must be strictly less than "
-                             "min(n_samples, n_features)=%r with "
-                             "svd_solver='%s'"
-                             % (n_components, min(n_samples, n_features),
-                                svd_solver))
+            raise ValueError(
+                "n_components=%r must be of type int "
+                "when greater than or equal to 1, was of type=%r"
+                % (n_components, type(n_components))
+            )
+        elif svd_solver == "arpack" and n_components == min(n_samples, n_features):
+            raise ValueError(
+                "n_components=%r must be strictly less than "
+                "min(n_samples, n_features)=%r with "
+                "svd_solver='%s'"
+                % (n_components, min(n_samples, n_features), svd_solver)
+            )
 
         random_state = check_random_state(self.random_state)
 
@@ -534,7 +592,7 @@ class PCA(_BasePCA):
         self.mean_ = np.mean(X, axis=0)
         X -= self.mean_
 
-        if svd_solver == 'arpack':
+        if svd_solver == "arpack":
             v0 = _init_arpack_v0(min(X.shape), random_state)
             U, S, Vt = svds(X, k=n_components, tol=self.tol, v0=v0)
             # svds doesn't abide by scipy.linalg.svd/randomized_svd
@@ -543,30 +601,40 @@ class PCA(_BasePCA):
             # flip eigenvectors' sign to enforce deterministic output
             U, Vt = svd_flip(U[:, ::-1], Vt[::-1])
 
-        elif svd_solver == 'randomized':
+        elif svd_solver == "randomized":
             # sign flipping is done inside
-            U, S, Vt = randomized_svd(X, n_components=n_components,
-                                      n_iter=self.iterated_power,
-                                      flip_sign=True,
-                                      random_state=random_state)
+            U, S, Vt = randomized_svd(
+                X,
+                n_components=n_components,
+                n_oversamples=self.n_oversamples,
+                n_iter=self.iterated_power,
+                power_iteration_normalizer=self.power_iteration_normalizer,
+                flip_sign=True,
+                random_state=random_state,
+            )
 
         self.n_samples_, self.n_features_ = n_samples, n_features
         self.components_ = Vt
         self.n_components_ = n_components
 
         # Get variance explained by singular values
-        self.explained_variance_ = (S ** 2) / (n_samples - 1)
-        total_var = np.var(X, ddof=1, axis=0)
-        self.explained_variance_ratio_ = \
-            self.explained_variance_ / total_var.sum()
+        self.explained_variance_ = (S**2) / (n_samples - 1)
+
+        # Workaround in-place variance calculation since at the time numpy
+        # did not have a way to calculate variance in-place.
+        N = X.shape[0] - 1
+        np.square(X, out=X)
+        np.sum(X, axis=0, out=X[0])
+        total_var = (X[0] / N).sum()
+
+        self.explained_variance_ratio_ = self.explained_variance_ / total_var
         self.singular_values_ = S.copy()  # Store the singular values.
 
         if self.n_components_ < min(n_features, n_samples):
-            self.noise_variance_ = (total_var.sum() -
-                                    self.explained_variance_.sum())
+            self.noise_variance_ = total_var - self.explained_variance_.sum()
             self.noise_variance_ /= min(n_features, n_samples) - n_components
         else:
-            self.noise_variance_ = 0.
+            self.noise_variance_ = 0.0
 
         return U, S, Vt
 
@@ -593,9 +661,8 @@ class PCA(_BasePCA):
         Xr = X - self.mean_
         n_features = X.shape[1]
         precision = self.get_precision()
-        log_like = -.5 * (Xr * (np.dot(Xr, precision))).sum(axis=1)
-        log_like -= .5 * (n_features * log(2. * np.pi) -
-                          fast_logdet(precision))
+        log_like = -0.5 * (Xr * (np.dot(Xr, precision))).sum(axis=1)
+        log_like -= 0.5 * (n_features * log(2.0 * np.pi) - fast_logdet(precision))
         return log_like
 
     def score(self, X, y=None):
@@ -611,6 +678,7 @@ class PCA(_BasePCA):
             The data.
 
         y : Ignored
+            Ignored.
 
         Returns
         -------
@@ -620,4 +688,4 @@ class PCA(_BasePCA):
         return np.mean(self.score_samples(X))
 
     def _more_tags(self):
-        return {'preserves_dtype': [np.float64, np.float32]}
+        return {"preserves_dtype": [np.float64, np.float32]}
