@@ -21,11 +21,23 @@ from .utils.multiclass import class_distribution
 
 
 class DummyClassifier(MultiOutputMixin, ClassifierMixin, BaseEstimator):
-    """
-    DummyClassifier is a classifier that makes predictions using simple rules.
+    """DummyClassifier makes predictions that ignore the input features.
 
-    This classifier is useful as a simple baseline to compare with other
-    (real) classifiers. Do not use it for real problems.
+    This classifier serves as a simple baseline to compare against other more
+    complex classifiers.
+
+    The specific behavior of the baseline is selected with the `strategy`
+    parameter.
+
+    All strategies make predictions that ignore the input feature values passed
+    as the `X` argument to `fit` and `predict`. The predictions, however,
+    typically depend on values observed in the `y` parameter passed to `fit`.
+
+    Note that the "stratified" and "uniform" strategies lead to
+    non-deterministic predictions that can be rendered deterministic by setting
+    the `random_state` parameter if needed. The other strategies are naturally
+    deterministic and, once fit, always return a the same constant prediction
+    for any value of `X`.
 
     Read more in the :ref:`User Guide <dummy_estimators>`.
 
@@ -33,20 +45,32 @@ class DummyClassifier(MultiOutputMixin, ClassifierMixin, BaseEstimator):
 
     Parameters
     ----------
-    strategy : {"stratified", "most_frequent", "prior", "uniform", \
+    strategy : {"most_frequent", "prior", "stratified", "uniform", \
             "constant"}, default="prior"
         Strategy to use to generate predictions.
 
-        * "stratified": generates predictions by respecting the training
-          set's class distribution.
-        * "most_frequent": always predicts the most frequent label in the
-          training set.
-        * "prior": always predicts the class that maximizes the class prior
-          (like "most_frequent") and ``predict_proba`` returns the class prior.
-        * "uniform": generates predictions uniformly at random.
+        * "most_frequent": the `predict` method always returns the most
+          frequent class label in the observed `y` argument passed to `fit`.
+          The `predict_proba` method returns the matching one-hot encoded
+          vector.
+        * "prior": the `predict` method always returns the most frequent
+          class label in the observed `y` argument passed to `fit` (like
+          "most_frequent"). ``predict_proba`` always returns the empirical
+          class distribution of `y` also known as the empirical class prior
+          distribution.
+        * "stratified": the `predict_proba` method randomly samples one-hot
+          vectors from a multinomial distribution parametrized by the empirical
+          class prior probabilities.
+          The `predict` method returns the class label which got probability
+          one in the one-hot vector of `predict_proba`.
+          Each sampled row of both methods is therefore independent and
+          identically distributed.
+        * "uniform": generates predictions uniformly at random from the list
+          of unique classes observed in `y`, i.e. each class has equal
+          probability.
         * "constant": always predicts a constant label that is provided by
           the user. This is useful for metrics that evaluate a non-majority
-          class
+          class.
 
           .. versionchanged:: 0.24
              The default value of `strategy` has changed to "prior" in version
@@ -58,20 +82,23 @@ class DummyClassifier(MultiOutputMixin, ClassifierMixin, BaseEstimator):
         Pass an int for reproducible output across multiple function calls.
         See :term:`Glossary <random_state>`.
 
-    constant : int or str or array-like of shape (n_outputs,)
+    constant : int or str or array-like of shape (n_outputs,), default=None
         The explicit constant as predicted by the "constant" strategy. This
         parameter is useful only for the "constant" strategy.
 
     Attributes
     ----------
     classes_ : ndarray of shape (n_classes,) or list of such arrays
-        Class labels for each output.
+        Unique class labels observed in `y`. For multi-output classification
+        problems, this attribute is a list of arrays as each output has an
+        independent set of possible classes.
 
     n_classes_ : int or list of int
         Number of label for each output.
 
     class_prior_ : ndarray of shape (n_classes,) or list of such arrays
-        Probability of each class for each output.
+        Frequency of each class observed in `y`. For multioutput classification
+        problems, this is computed independently for each output.
 
     n_outputs_ : int
         Number of outputs.
@@ -85,7 +112,8 @@ class DummyClassifier(MultiOutputMixin, ClassifierMixin, BaseEstimator):
 
     sparse_output_ : bool
         True if the array returned from predict is to be in sparse CSC format.
-        Is automatically set to True if the input y is passed in sparse format.
+        Is automatically set to True if the input `y` is passed in sparse
+        format.
 
     See Also
     --------
@@ -112,7 +140,7 @@ class DummyClassifier(MultiOutputMixin, ClassifierMixin, BaseEstimator):
         self.constant = constant
 
     def fit(self, X, y, sample_weight=None):
-        """Fit the random classifier.
+        """Fit the baseline classifier.
 
         Parameters
         ----------
@@ -530,7 +558,7 @@ class DummyRegressor(MultiOutputMixin, RegressorMixin, BaseEstimator):
                 % (self.strategy, allowed_strategies)
             )
 
-        y = check_array(y, ensure_2d=False)
+        y = check_array(y, ensure_2d=False, input_name="y")
         if len(y) == 0:
             raise ValueError("y must not be empty.")
 
@@ -578,20 +606,18 @@ class DummyRegressor(MultiOutputMixin, RegressorMixin, BaseEstimator):
                     "when the constant strategy is used."
                 )
 
-            self.constant = check_array(
+            self.constant_ = check_array(
                 self.constant,
                 accept_sparse=["csr", "csc", "coo"],
                 ensure_2d=False,
                 ensure_min_samples=0,
             )
 
-            if self.n_outputs_ != 1 and self.constant.shape[0] != y.shape[1]:
+            if self.n_outputs_ != 1 and self.constant_.shape[0] != y.shape[1]:
                 raise ValueError(
                     "Constant target value should have shape (%d, 1)." % y.shape[1]
                 )
 
-            self.constant_ = self.constant
-
         self.constant_ = np.reshape(self.constant_, (1, -1))
         return self
 
