@@ -12,14 +12,12 @@ import scipy.optimize
 from scipy.special import erf, expit
 
 from ..base import BaseEstimator, ClassifierMixin, clone
-from .kernels \
-    import RBF, CompoundKernel, ConstantKernel as C
-from ..utils.validation import check_is_fitted, check_array
+from .kernels import RBF, CompoundKernel, ConstantKernel as C
+from ..utils.validation import check_is_fitted
 from ..utils import check_random_state
 from ..utils.optimize import _check_optimize_result
 from ..preprocessing import LabelEncoder
 from ..multiclass import OneVsRestClassifier, OneVsOneClassifier
-from ..utils.validation import _deprecate_positional_args
 
 
 # Values required for approximating the logistic sigmoid by
@@ -29,8 +27,9 @@ from ..utils.validation import _deprecate_positional_args
 # A = (erf(np.dot(x, self.lambdas)) + 1) / 2
 # coefs = lstsq(A, b)[0]
 LAMBDAS = np.array([0.41, 0.4, 0.37, 0.44, 0.39])[:, np.newaxis]
-COEFS = np.array([-1854.8214151, 3516.89893646, 221.29346712,
-                  128.12323805, -2010.49422654])[:, np.newaxis]
+COEFS = np.array(
+    [-1854.8214151, 3516.89893646, 221.29346712, 128.12323805, -2010.49422654]
+)[:, np.newaxis]
 
 
 class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
@@ -111,7 +110,7 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
     random_state : int, RandomState instance or None, default=None
         Determines random number generation used to initialize the centers.
         Pass an int for reproducible results across multiple function calls.
-        See :term: `Glossary <random_state>`.
+        See :term:`Glossary <random_state>`.
 
     Attributes
     ----------
@@ -145,10 +144,18 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
         The log-marginal-likelihood of ``self.kernel_.theta``
 
     """
-    @_deprecate_positional_args
-    def __init__(self, kernel=None, *, optimizer="fmin_l_bfgs_b",
-                 n_restarts_optimizer=0, max_iter_predict=100,
-                 warm_start=False, copy_X_train=True, random_state=None):
+
+    def __init__(
+        self,
+        kernel=None,
+        *,
+        optimizer="fmin_l_bfgs_b",
+        n_restarts_optimizer=0,
+        max_iter_predict=100,
+        warm_start=False,
+        copy_X_train=True,
+        random_state=None,
+    ):
         self.kernel = kernel
         self.optimizer = optimizer
         self.n_restarts_optimizer = n_restarts_optimizer
@@ -158,7 +165,7 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
         self.random_state = random_state
 
     def fit(self, X, y):
-        """Fit Gaussian process classification model
+        """Fit Gaussian process classification model.
 
         Parameters
         ----------
@@ -166,15 +173,16 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
             Feature vectors or other representations of training data.
 
         y : array-like of shape (n_samples,)
-            Target values, must be binary
+            Target values, must be binary.
 
         Returns
         -------
         self : returns an instance of self.
         """
         if self.kernel is None:  # Use an RBF kernel as default
-            self.kernel_ = C(1.0, constant_value_bounds="fixed") \
-                * RBF(1.0, length_scale_bounds="fixed")
+            self.kernel_ = C(1.0, constant_value_bounds="fixed") * RBF(
+                1.0, length_scale_bounds="fixed"
+            )
         else:
             self.kernel_ = clone(self.kernel)
 
@@ -188,13 +196,16 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
         self.y_train_ = label_encoder.fit_transform(y)
         self.classes_ = label_encoder.classes_
         if self.classes_.size > 2:
-            raise ValueError("%s supports only binary classification. "
-                             "y contains classes %s"
-                             % (self.__class__.__name__, self.classes_))
+            raise ValueError(
+                "%s supports only binary classification. y contains classes %s"
+                % (self.__class__.__name__, self.classes_)
+            )
         elif self.classes_.size == 1:
-            raise ValueError("{0:s} requires 2 classes; got {1:d} class"
-                             .format(self.__class__.__name__,
-                                     self.classes_.size))
+            raise ValueError(
+                "{0:s} requires 2 classes; got {1:d} class".format(
+                    self.__class__.__name__, self.classes_.size
+                )
+            )
 
         if self.optimizer is not None and self.kernel_.n_dims > 0:
             # Choose hyperparameters based on maximizing the log-marginal
@@ -202,16 +213,18 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
             def obj_func(theta, eval_gradient=True):
                 if eval_gradient:
                     lml, grad = self.log_marginal_likelihood(
-                        theta, eval_gradient=True, clone_kernel=False)
+                        theta, eval_gradient=True, clone_kernel=False
+                    )
                     return -lml, -grad
                 else:
-                    return -self.log_marginal_likelihood(theta,
-                                                         clone_kernel=False)
+                    return -self.log_marginal_likelihood(theta, clone_kernel=False)
 
             # First optimize starting from theta specified in kernel
-            optima = [self._constrained_optimization(obj_func,
-                                                     self.kernel_.theta,
-                                                     self.kernel_.bounds)]
+            optima = [
+                self._constrained_optimization(
+                    obj_func, self.kernel_.theta, self.kernel_.bounds
+                )
+            ]
 
             # Additional runs are performed from log-uniform chosen initial
             # theta
@@ -219,14 +232,14 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
                 if not np.isfinite(self.kernel_.bounds).all():
                     raise ValueError(
                         "Multiple optimizer restarts (n_restarts_optimizer>0) "
-                        "requires that all bounds are finite.")
+                        "requires that all bounds are finite."
+                    )
                 bounds = self.kernel_.bounds
                 for iteration in range(self.n_restarts_optimizer):
-                    theta_initial = np.exp(self.rng.uniform(bounds[:, 0],
-                                                            bounds[:, 1]))
+                    theta_initial = np.exp(self.rng.uniform(bounds[:, 0], bounds[:, 1]))
                     optima.append(
-                        self._constrained_optimization(obj_func, theta_initial,
-                                                       bounds))
+                        self._constrained_optimization(obj_func, theta_initial, bounds)
+                    )
             # Select result from run with minimal (negative) log-marginal
             # likelihood
             lml_values = list(map(itemgetter(1), optima))
@@ -235,15 +248,17 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
 
             self.log_marginal_likelihood_value_ = -np.min(lml_values)
         else:
-            self.log_marginal_likelihood_value_ = \
-                self.log_marginal_likelihood(self.kernel_.theta)
+            self.log_marginal_likelihood_value_ = self.log_marginal_likelihood(
+                self.kernel_.theta
+            )
 
         # Precompute quantities required for predictions which are independent
         # of actual query points
         K = self.kernel_(self.X_train_)
 
-        _, (self.pi_, self.W_sr_, self.L_, _, _) = \
-            self._posterior_mode(K, return_temporaries=True)
+        _, (self.pi_, self.W_sr_, self.L_, _, _) = self._posterior_mode(
+            K, return_temporaries=True
+        )
 
         return self
 
@@ -303,15 +318,18 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
         # blitiri.blogspot.de/2012/11/gaussian-integral-of-error-function.html
         alpha = 1 / (2 * var_f_star)
         gamma = LAMBDAS * f_star
-        integrals = np.sqrt(np.pi / alpha) \
-            * erf(gamma * np.sqrt(alpha / (alpha + LAMBDAS**2))) \
+        integrals = (
+            np.sqrt(np.pi / alpha)
+            * erf(gamma * np.sqrt(alpha / (alpha + LAMBDAS**2)))
             / (2 * np.sqrt(var_f_star * 2 * np.pi))
-        pi_star = (COEFS * integrals).sum(axis=0) + .5 * COEFS.sum()
+        )
+        pi_star = (COEFS * integrals).sum(axis=0) + 0.5 * COEFS.sum()
 
         return np.vstack((1 - pi_star, pi_star)).T
 
-    def log_marginal_likelihood(self, theta=None, eval_gradient=False,
-                                clone_kernel=True):
+    def log_marginal_likelihood(
+        self, theta=None, eval_gradient=False, clone_kernel=True
+    ):
         """Returns log-marginal likelihood of theta for training data.
 
         Parameters
@@ -343,8 +361,7 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
         """
         if theta is None:
             if eval_gradient:
-                raise ValueError(
-                    "Gradient can only be evaluated for theta!=None")
+                raise ValueError("Gradient can only be evaluated for theta!=None")
             return self.log_marginal_likelihood_value_
 
         if clone_kernel:
@@ -360,8 +377,7 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
 
         # Compute log-marginal-likelihood Z and also store some temporaries
         # which can be reused for computing Z's gradient
-        Z, (pi, W_sr, L, b, a) = \
-            self._posterior_mode(K, return_temporaries=True)
+        Z, (pi, W_sr, L, b, a) = self._posterior_mode(K, return_temporaries=True)
 
         if not eval_gradient:
             return Z
@@ -372,13 +388,16 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
         R = W_sr[:, np.newaxis] * cho_solve((L, True), np.diag(W_sr))  # Line 7
         C = solve(L, W_sr[:, np.newaxis] * K)  # Line 8
         # Line 9: (use einsum to compute np.diag(C.T.dot(C))))
-        s_2 = -0.5 * (np.diag(K) - np.einsum('ij, ij -> j', C, C)) \
-            * (pi * (1 - pi) * (1 - 2 * pi))  # third derivative
+        s_2 = (
+            -0.5
+            * (np.diag(K) - np.einsum("ij, ij -> j", C, C))
+            * (pi * (1 - pi) * (1 - 2 * pi))
+        )  # third derivative
 
         for j in range(d_Z.shape[0]):
-            C = K_gradient[:, :, j]   # Line 11
+            C = K_gradient[:, :, j]  # Line 11
             # Line 12: (R.T.ravel().dot(C.ravel()) = np.trace(R.dot(C)))
-            s_1 = .5 * a.T.dot(C).dot(a) - .5 * R.T.ravel().dot(C.ravel())
+            s_1 = 0.5 * a.T.dot(C).dot(a) - 0.5 * R.T.ravel().dot(C.ravel())
 
             b = C.dot(self.y_train_ - pi)  # Line 13
             s_3 = b - K.dot(R.dot(b))  # Line 14
@@ -398,8 +417,11 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
 
         # If warm_start are enabled, we reuse the last solution for the
         # posterior mode as initialization; otherwise, we initialize with 0
-        if self.warm_start and hasattr(self, "f_cached") \
-           and self.f_cached.shape == self.y_train_.shape:
+        if (
+            self.warm_start
+            and hasattr(self, "f_cached")
+            and self.f_cached.shape == self.y_train_.shape
+        ):
             f = self.f_cached
         else:
             f = np.zeros_like(self.y_train_, dtype=np.float64)
@@ -424,9 +446,11 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
 
             # Line 10: Compute log marginal likelihood in loop and use as
             #          convergence criterion
-            lml = -0.5 * a.T.dot(f) \
-                - np.log1p(np.exp(-(self.y_train_ * 2 - 1) * f)).sum() \
+            lml = (
+                -0.5 * a.T.dot(f)
+                - np.log1p(np.exp(-(self.y_train_ * 2 - 1) * f)).sum()
                 - np.log(np.diag(L)).sum()
+            )
             # Check if we have converged (log marginal likelihood does
             # not decrease)
             # XXX: more complex convergence criterion
@@ -443,13 +467,12 @@ class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
     def _constrained_optimization(self, obj_func, initial_theta, bounds):
         if self.optimizer == "fmin_l_bfgs_b":
             opt_res = scipy.optimize.minimize(
-                obj_func, initial_theta, method="L-BFGS-B", jac=True,
-                bounds=bounds)
+                obj_func, initial_theta, method="L-BFGS-B", jac=True, bounds=bounds
+            )
             _check_optimize_result("lbfgs", opt_res)
             theta_opt, func_min = opt_res.x, opt_res.fun
         elif callable(self.optimizer):
-            theta_opt, func_min = \
-                self.optimizer(obj_func, initial_theta, bounds=bounds)
+            theta_opt, func_min = self.optimizer(obj_func, initial_theta, bounds=bounds)
         else:
             raise ValueError("Unknown optimizer %s." % self.optimizer)
 
@@ -473,12 +496,15 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
 
     Read more in the :ref:`User Guide <gaussian_process>`.
 
+    .. versionadded:: 0.18
+
     Parameters
     ----------
     kernel : kernel instance, default=None
         The kernel specifying the covariance function of the GP. If None is
         passed, the kernel "1.0 * RBF(1.0)" is used as default. Note that
-        the kernel's hyperparameters are optimized during fitting.
+        the kernel's hyperparameters are optimized during fitting. Also kernel
+        cannot be a `CompoundKernel`.
 
     optimizer : 'fmin_l_bfgs_b' or callable, default='fmin_l_bfgs_b'
         Can either be one of the internally supported optimizers for optimizing
@@ -536,7 +562,7 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
     random_state : int, RandomState instance or None, default=None
         Determines random number generation used to initialize the centers.
         Pass an int for reproducible results across multiple function calls.
-        See :term: `Glossary <random_state>`.
+        See :term:`Glossary <random_state>`.
 
     multi_class : {'one_vs_rest', 'one_vs_one'}, default='one_vs_rest'
         Specifies how multi-class classification problems are handled.
@@ -578,6 +604,21 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
     n_classes_ : int
         The number of classes in the training data
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
+    See Also
+    --------
+    GaussianProcessRegressor : Gaussian process regression (GPR).
+
     Examples
     --------
     >>> from sklearn.datasets import load_iris
@@ -592,14 +633,21 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
     >>> gpc.predict_proba(X[:2,:])
     array([[0.83548752, 0.03228706, 0.13222543],
            [0.79064206, 0.06525643, 0.14410151]])
-
-    .. versionadded:: 0.18
     """
-    @_deprecate_positional_args
-    def __init__(self, kernel=None, *, optimizer="fmin_l_bfgs_b",
-                 n_restarts_optimizer=0, max_iter_predict=100,
-                 warm_start=False, copy_X_train=True, random_state=None,
-                 multi_class="one_vs_rest", n_jobs=None):
+
+    def __init__(
+        self,
+        kernel=None,
+        *,
+        optimizer="fmin_l_bfgs_b",
+        n_restarts_optimizer=0,
+        max_iter_predict=100,
+        warm_start=False,
+        copy_X_train=True,
+        random_state=None,
+        multi_class="one_vs_rest",
+        n_jobs=None,
+    ):
         self.kernel = kernel
         self.optimizer = optimizer
         self.n_restarts_optimizer = n_restarts_optimizer
@@ -611,7 +659,7 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
         self.n_jobs = n_jobs
 
     def fit(self, X, y):
-        """Fit Gaussian process classification model
+        """Fit Gaussian process classification model.
 
         Parameters
         ----------
@@ -619,18 +667,24 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
             Feature vectors or other representations of training data.
 
         y : array-like of shape (n_samples,)
-            Target values, must be binary
+            Target values, must be binary.
 
         Returns
         -------
-        self : returns an instance of self.
+        self : object
+            Returns an instance of self.
         """
+        if isinstance(self.kernel, CompoundKernel):
+            raise ValueError("kernel cannot be a CompoundKernel")
+
         if self.kernel is None or self.kernel.requires_vector_input:
-            X, y = self._validate_data(X, y, multi_output=False,
-                                       ensure_2d=True, dtype="numeric")
+            X, y = self._validate_data(
+                X, y, multi_output=False, ensure_2d=True, dtype="numeric"
+            )
         else:
-            X, y = self._validate_data(X, y, multi_output=False,
-                                       ensure_2d=False, dtype=None)
+            X, y = self._validate_data(
+                X, y, multi_output=False, ensure_2d=False, dtype=None
+            )
 
         self.base_estimator_ = _BinaryGaussianProcessClassifierLaplace(
             kernel=self.kernel,
@@ -639,37 +693,42 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
             max_iter_predict=self.max_iter_predict,
             warm_start=self.warm_start,
             copy_X_train=self.copy_X_train,
-            random_state=self.random_state)
+            random_state=self.random_state,
+        )
 
         self.classes_ = np.unique(y)
         self.n_classes_ = self.classes_.size
         if self.n_classes_ == 1:
-            raise ValueError("GaussianProcessClassifier requires 2 or more "
-                             "distinct classes; got %d class (only class %s "
-                             "is present)"
-                             % (self.n_classes_, self.classes_[0]))
+            raise ValueError(
+                "GaussianProcessClassifier requires 2 or more "
+                "distinct classes; got %d class (only class %s "
+                "is present)" % (self.n_classes_, self.classes_[0])
+            )
         if self.n_classes_ > 2:
             if self.multi_class == "one_vs_rest":
-                self.base_estimator_ = \
-                    OneVsRestClassifier(self.base_estimator_,
-                                        n_jobs=self.n_jobs)
+                self.base_estimator_ = OneVsRestClassifier(
+                    self.base_estimator_, n_jobs=self.n_jobs
+                )
             elif self.multi_class == "one_vs_one":
-                self.base_estimator_ = \
-                    OneVsOneClassifier(self.base_estimator_,
-                                       n_jobs=self.n_jobs)
+                self.base_estimator_ = OneVsOneClassifier(
+                    self.base_estimator_, n_jobs=self.n_jobs
+                )
             else:
-                raise ValueError("Unknown multi-class mode %s"
-                                 % self.multi_class)
+                raise ValueError("Unknown multi-class mode %s" % self.multi_class)
 
         self.base_estimator_.fit(X, y)
 
         if self.n_classes_ > 2:
             self.log_marginal_likelihood_value_ = np.mean(
-                [estimator.log_marginal_likelihood()
-                 for estimator in self.base_estimator_.estimators_])
+                [
+                    estimator.log_marginal_likelihood()
+                    for estimator in self.base_estimator_.estimators_
+                ]
+            )
         else:
-            self.log_marginal_likelihood_value_ = \
+            self.log_marginal_likelihood_value_ = (
                 self.base_estimator_.log_marginal_likelihood()
+            )
 
         return self
 
@@ -684,14 +743,14 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
         Returns
         -------
         C : ndarray of shape (n_samples,)
-            Predicted target values for X, values are from ``classes_``
+            Predicted target values for X, values are from ``classes_``.
         """
         check_is_fitted(self)
 
         if self.kernel is None or self.kernel.requires_vector_input:
-            X = check_array(X, ensure_2d=True, dtype="numeric")
+            X = self._validate_data(X, ensure_2d=True, dtype="numeric", reset=False)
         else:
-            X = check_array(X, ensure_2d=False, dtype=None)
+            X = self._validate_data(X, ensure_2d=False, dtype=None, reset=False)
 
         return self.base_estimator_.predict(X)
 
@@ -712,29 +771,33 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
         """
         check_is_fitted(self)
         if self.n_classes_ > 2 and self.multi_class == "one_vs_one":
-            raise ValueError("one_vs_one multi-class mode does not support "
-                             "predicting probability estimates. Use "
-                             "one_vs_rest mode instead.")
+            raise ValueError(
+                "one_vs_one multi-class mode does not support "
+                "predicting probability estimates. Use "
+                "one_vs_rest mode instead."
+            )
 
         if self.kernel is None or self.kernel.requires_vector_input:
-            X = check_array(X, ensure_2d=True, dtype="numeric")
+            X = self._validate_data(X, ensure_2d=True, dtype="numeric", reset=False)
         else:
-            X = check_array(X, ensure_2d=False, dtype=None)
+            X = self._validate_data(X, ensure_2d=False, dtype=None, reset=False)
 
         return self.base_estimator_.predict_proba(X)
 
     @property
     def kernel_(self):
+        """Return the kernel of the base estimator."""
         if self.n_classes_ == 2:
             return self.base_estimator_.kernel_
         else:
             return CompoundKernel(
-                [estimator.kernel_
-                 for estimator in self.base_estimator_.estimators_])
+                [estimator.kernel_ for estimator in self.base_estimator_.estimators_]
+            )
 
-    def log_marginal_likelihood(self, theta=None, eval_gradient=False,
-                                clone_kernel=True):
-        """Returns log-marginal likelihood of theta for training data.
+    def log_marginal_likelihood(
+        self, theta=None, eval_gradient=False, clone_kernel=True
+    ):
+        """Return log-marginal likelihood of theta for training data.
 
         In the case of multi-class classification, the mean log-marginal
         likelihood of the one-versus-rest classifiers are returned.
@@ -773,35 +836,45 @@ class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
 
         if theta is None:
             if eval_gradient:
-                raise ValueError(
-                    "Gradient can only be evaluated for theta!=None")
+                raise ValueError("Gradient can only be evaluated for theta!=None")
             return self.log_marginal_likelihood_value_
 
         theta = np.asarray(theta)
         if self.n_classes_ == 2:
             return self.base_estimator_.log_marginal_likelihood(
-                theta, eval_gradient, clone_kernel=clone_kernel)
+                theta, eval_gradient, clone_kernel=clone_kernel
+            )
         else:
             if eval_gradient:
                 raise NotImplementedError(
                     "Gradient of log-marginal-likelihood not implemented for "
-                    "multi-class GPC.")
+                    "multi-class GPC."
+                )
             estimators = self.base_estimator_.estimators_
             n_dims = estimators[0].kernel_.n_dims
             if theta.shape[0] == n_dims:  # use same theta for all sub-kernels
                 return np.mean(
-                    [estimator.log_marginal_likelihood(
-                        theta, clone_kernel=clone_kernel)
-                     for i, estimator in enumerate(estimators)])
+                    [
+                        estimator.log_marginal_likelihood(
+                            theta, clone_kernel=clone_kernel
+                        )
+                        for i, estimator in enumerate(estimators)
+                    ]
+                )
             elif theta.shape[0] == n_dims * self.classes_.shape[0]:
                 # theta for compound kernel
                 return np.mean(
-                    [estimator.log_marginal_likelihood(
-                        theta[n_dims * i:n_dims * (i + 1)],
-                        clone_kernel=clone_kernel)
-                     for i, estimator in enumerate(estimators)])
+                    [
+                        estimator.log_marginal_likelihood(
+                            theta[n_dims * i : n_dims * (i + 1)],
+                            clone_kernel=clone_kernel,
+                        )
+                        for i, estimator in enumerate(estimators)
+                    ]
+                )
             else:
-                raise ValueError("Shape of theta must be either %d or %d. "
-                                 "Obtained theta with shape %d."
-                                 % (n_dims, n_dims * self.classes_.shape[0],
-                                    theta.shape[0]))
+                raise ValueError(
+                    "Shape of theta must be either %d or %d. "
+                    "Obtained theta with shape %d."
+                    % (n_dims, n_dims * self.classes_.shape[0], theta.shape[0])
+                )
