@@ -22,15 +22,18 @@ from abc import ABCMeta, abstractmethod
 from .base import BaseEstimator, clone, MetaEstimatorMixin
 from .base import RegressorMixin, ClassifierMixin, is_classifier
 from .model_selection import cross_val_predict
-from .utils import check_array, check_X_y, check_random_state
-from .utils.metaestimators import if_delegate_has_method
-from .utils.validation import (check_is_fitted, has_fit_parameter,
-                               _check_fit_params, _deprecate_positional_args)
+from .utils.metaestimators import available_if
+from .utils import check_random_state
+from .utils.validation import check_is_fitted, has_fit_parameter, _check_fit_params
 from .utils.multiclass import check_classification_targets
 from .utils.fixes import delayed
 
-__all__ = ["MultiOutputRegressor", "MultiOutputClassifier",
-           "ClassifierChain", "RegressorChain"]
+__all__ = [
+    "MultiOutputRegressor",
+    "MultiOutputClassifier",
+    "ClassifierChain",
+    "RegressorChain",
+]
 
 
 def _fit_estimator(estimator, X, y, sample_weight=None, **fit_params):
@@ -42,15 +45,15 @@ def _fit_estimator(estimator, X, y, sample_weight=None, **fit_params):
     return estimator
 
 
-def _partial_fit_estimator(estimator, X, y, classes=None, sample_weight=None,
-                           first_time=True):
+def _partial_fit_estimator(
+    estimator, X, y, classes=None, sample_weight=None, first_time=True
+):
     if first_time:
         estimator = clone(estimator)
 
     if sample_weight is not None:
         if classes is not None:
-            estimator.partial_fit(X, y, classes=classes,
-                                  sample_weight=sample_weight)
+            estimator.partial_fit(X, y, classes=classes, sample_weight=sample_weight)
         else:
             estimator.partial_fit(X, y, sample_weight=sample_weight)
     else:
@@ -61,85 +64,106 @@ def _partial_fit_estimator(estimator, X, y, classes=None, sample_weight=None,
     return estimator
 
 
-class _MultiOutputEstimator(MetaEstimatorMixin,
-                            BaseEstimator,
-                            metaclass=ABCMeta):
+def _available_if_estimator_has(attr):
+    """Return a function to check if `estimator` or `estimators_` has `attr`.
+
+    Helper for Chain implementations.
+    """
+
+    def _check(self):
+        return hasattr(self.estimator, attr) or all(
+            hasattr(est, attr) for est in self.estimators_
+        )
+
+    return available_if(_check)
+
+
+class _MultiOutputEstimator(MetaEstimatorMixin, BaseEstimator, metaclass=ABCMeta):
     @abstractmethod
-    @_deprecate_positional_args
     def __init__(self, estimator, *, n_jobs=None):
         self.estimator = estimator
         self.n_jobs = n_jobs
 
-    @if_delegate_has_method('estimator')
+    @_available_if_estimator_has("partial_fit")
     def partial_fit(self, X, y, classes=None, sample_weight=None):
-        """Incrementally fit the model to data.
-        Fit a separate model for each output variable.
+        """Incrementally fit a separate model for each class output.
 
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Data.
+            The input data.
 
         y : {array-like, sparse matrix} of shape (n_samples, n_outputs)
             Multi-output targets.
 
-        classes : list of ndarray of shape (n_outputs,)
-            Each array is unique classes for one output in str/int
-            Can be obtained by via
-            ``[np.unique(y[:, i]) for i in range(y.shape[1])]``, where y is the
-            target matrix of the entire dataset.
+        classes : list of ndarray of shape (n_outputs,), default=None
+            Each array is unique classes for one output in str/int.
+            Can be obtained via
+            ``[np.unique(y[:, i]) for i in range(y.shape[1])]``, where `y`
+            is the target matrix of the entire dataset.
             This argument is required for the first call to partial_fit
             and can be omitted in the subsequent calls.
-            Note that y doesn't need to contain all labels in `classes`.
+            Note that `y` doesn't need to contain all labels in `classes`.
 
         sample_weight : array-like of shape (n_samples,), default=None
-            Sample weights. If None, then samples are equally weighted.
+            Sample weights. If `None`, then samples are equally weighted.
             Only supported if the underlying regressor supports sample
             weights.
 
         Returns
         -------
         self : object
+            Returns a fitted instance.
         """
-        X, y = check_X_y(X, y,
-                         force_all_finite=False,
-                         multi_output=True,
-                         accept_sparse=True)
+        first_time = not hasattr(self, "estimators_")
+        y = self._validate_data(X="no_validation", y=y, multi_output=True)
 
         if y.ndim == 1:
-            raise ValueError("y must have at least two dimensions for "
-                             "multi-output regression but has only one.")
+            raise ValueError(
+                "y must have at least two dimensions for "
+                "multi-output regression but has only one."
+            )
 
-        if (sample_weight is not None and
-                not has_fit_parameter(self.estimator, 'sample_weight')):
-            raise ValueError("Underlying estimator does not support"
-                             " sample weights.")
+        if sample_weight is not None and not has_fit_parameter(
+            self.estimator, "sample_weight"
+        ):
+            raise ValueError("Underlying estimator does not support sample weights.")
 
-        first_time = not hasattr(self, 'estimators_')
+        first_time = not hasattr(self, "estimators_")
 
         self.estimators_ = Parallel(n_jobs=self.n_jobs)(
             delayed(_partial_fit_estimator)(
                 self.estimators_[i] if not first_time else self.estimator,
-                X, y[:, i],
+                X,
+                y[:, i],
                 classes[i] if classes is not None else None,
-                sample_weight, first_time) for i in range(y.shape[1]))
+                sample_weight,
+                first_time,
+            )
+            for i in range(y.shape[1])
+        )
+
+        if first_time and hasattr(self.estimators_[0], "n_features_in_"):
+            self.n_features_in_ = self.estimators_[0].n_features_in_
+        if first_time and hasattr(self.estimators_[0], "feature_names_in_"):
+            self.feature_names_in_ = self.estimators_[0].feature_names_in_
+
         return self
 
     def fit(self, X, y, sample_weight=None, **fit_params):
-        """ Fit the model to data.
-        Fit a separate model for each output variable.
+        """Fit the model to data, separately for each output variable.
 
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Data.
+            The input data.
 
         y : {array-like, sparse matrix} of shape (n_samples, n_outputs)
             Multi-output targets. An indicator matrix turns on multilabel
             estimation.
 
         sample_weight : array-like of shape (n_samples,), default=None
-            Sample weights. If None, then samples are equally weighted.
+            Sample weights. If `None`, then samples are equally weighted.
             Only supported if the underlying regressor supports sample
             weights.
 
@@ -151,45 +175,51 @@ class _MultiOutputEstimator(MetaEstimatorMixin,
         Returns
         -------
         self : object
+            Returns a fitted instance.
         """
 
         if not hasattr(self.estimator, "fit"):
-            raise ValueError("The base estimator should implement"
-                             " a fit method")
+            raise ValueError("The base estimator should implement a fit method")
 
-        X, y = self._validate_data(X, y,
-                                   force_all_finite=False,
-                                   multi_output=True, accept_sparse=True)
+        y = self._validate_data(X="no_validation", y=y, multi_output=True)
 
         if is_classifier(self):
             check_classification_targets(y)
 
         if y.ndim == 1:
-            raise ValueError("y must have at least two dimensions for "
-                             "multi-output regression but has only one.")
+            raise ValueError(
+                "y must have at least two dimensions for "
+                "multi-output regression but has only one."
+            )
 
-        if (sample_weight is not None and
-                not has_fit_parameter(self.estimator, 'sample_weight')):
-            raise ValueError("Underlying estimator does not support"
-                             " sample weights.")
+        if sample_weight is not None and not has_fit_parameter(
+            self.estimator, "sample_weight"
+        ):
+            raise ValueError("Underlying estimator does not support sample weights.")
 
         fit_params_validated = _check_fit_params(X, fit_params)
 
         self.estimators_ = Parallel(n_jobs=self.n_jobs)(
             delayed(_fit_estimator)(
-                self.estimator, X, y[:, i], sample_weight,
-                **fit_params_validated)
-            for i in range(y.shape[1]))
+                self.estimator, X, y[:, i], sample_weight, **fit_params_validated
+            )
+            for i in range(y.shape[1])
+        )
+
+        if hasattr(self.estimators_[0], "n_features_in_"):
+            self.n_features_in_ = self.estimators_[0].n_features_in_
+        if hasattr(self.estimators_[0], "feature_names_in_"):
+            self.feature_names_in_ = self.estimators_[0].feature_names_in_
+
         return self
 
     def predict(self, X):
-        """Predict multi-output variable using a model
-         trained for each target variable.
+        """Predict multi-output variable using model for each target variable.
 
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Data.
+            The input data.
 
         Returns
         -------
@@ -198,24 +228,21 @@ class _MultiOutputEstimator(MetaEstimatorMixin,
             Note: Separate models are generated for each predictor.
         """
         check_is_fitted(self)
-        if not hasattr(self.estimator, "predict"):
-            raise ValueError("The base estimator should implement"
-                             " a predict method")
-
-        X = check_array(X, force_all_finite=False, accept_sparse=True)
+        if not hasattr(self.estimators_[0], "predict"):
+            raise ValueError("The base estimator should implement a predict method")
 
         y = Parallel(n_jobs=self.n_jobs)(
-            delayed(e.predict)(X)
-            for e in self.estimators_)
+            delayed(e.predict)(X) for e in self.estimators_
+        )
 
         return np.asarray(y).T
 
     def _more_tags(self):
-        return {'multioutput_only': True}
+        return {"multioutput_only": True}
 
 
 class MultiOutputRegressor(RegressorMixin, _MultiOutputEstimator):
-    """Multi target regression
+    """Multi target regression.
 
     This strategy consists of fitting one regressor per target. This is a
     simple strategy for extending regressors that do not natively support
@@ -237,18 +264,37 @@ class MultiOutputRegressor(RegressorMixin, _MultiOutputEstimator):
         using ``n_jobs > 1`` can result in slower performance due
         to the parallelism overhead.
 
-        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+        ``None`` means `1` unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all available processes / threads.
         See :term:`Glossary <n_jobs>` for more details.
 
         .. versionchanged:: 0.20
-           `n_jobs` default changed from 1 to None
+            `n_jobs` default changed from `1` to `None`.
 
     Attributes
     ----------
     estimators_ : list of ``n_output`` estimators
         Estimators used for predictions.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`. Only defined if the
+        underlying `estimator` exposes such an attribute when fit.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Only defined if the
+        underlying estimators expose such an attribute when fit.
+
+        .. versionadded:: 1.0
+
+    See Also
+    --------
+    RegressorChain : A multi-label model that arranges regressions into a
+        chain.
+    MultiOutputClassifier : Classifies each output independently rather than
+        chaining.
+
     Examples
     --------
     >>> import numpy as np
@@ -256,46 +302,45 @@ class MultiOutputRegressor(RegressorMixin, _MultiOutputEstimator):
     >>> from sklearn.multioutput import MultiOutputRegressor
     >>> from sklearn.linear_model import Ridge
     >>> X, y = load_linnerud(return_X_y=True)
-    >>> clf = MultiOutputRegressor(Ridge(random_state=123)).fit(X, y)
-    >>> clf.predict(X[[0]])
+    >>> regr = MultiOutputRegressor(Ridge(random_state=123)).fit(X, y)
+    >>> regr.predict(X[[0]])
     array([[176..., 35..., 57...]])
     """
-    @_deprecate_positional_args
+
     def __init__(self, estimator, *, n_jobs=None):
         super().__init__(estimator, n_jobs=n_jobs)
 
-    @if_delegate_has_method('estimator')
+    @_available_if_estimator_has("partial_fit")
     def partial_fit(self, X, y, sample_weight=None):
-        """Incrementally fit the model to data.
-        Fit a separate model for each output variable.
+        """Incrementally fit the model to data, for each output variable.
 
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
-            Data.
+            The input data.
 
         y : {array-like, sparse matrix} of shape (n_samples, n_outputs)
             Multi-output targets.
 
         sample_weight : array-like of shape (n_samples,), default=None
-            Sample weights. If None, then samples are equally weighted.
+            Sample weights. If `None`, then samples are equally weighted.
             Only supported if the underlying regressor supports sample
             weights.
 
         Returns
         -------
         self : object
+            Returns a fitted instance.
         """
-        super().partial_fit(
-            X, y, sample_weight=sample_weight)
+        super().partial_fit(X, y, sample_weight=sample_weight)
 
 
 class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
-    """Multi target classification
+    """Multi target classification.
 
     This strategy consists of fitting one classifier per target. This is a
     simple strategy for extending classifiers that do not natively support
-    multi-target classification
+    multi-target classification.
 
     Parameters
     ----------
@@ -312,12 +357,12 @@ class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
         using ``n_jobs > 1`` can result in slower performance due
         to the parallelism overhead.
 
-        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+        ``None`` means `1` unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all available processes / threads.
         See :term:`Glossary <n_jobs>` for more details.
 
         .. versionchanged:: 0.20
-           `n_jobs` default changed from 1 to None
+            `n_jobs` default changed from `1` to `None`.
 
     Attributes
     ----------
@@ -327,19 +372,37 @@ class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
     estimators_ : list of ``n_output`` estimators
         Estimators used for predictions.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`. Only defined if the
+        underlying `estimator` exposes such an attribute when fit.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Only defined if the
+        underlying estimators expose such an attribute when fit.
+
+        .. versionadded:: 1.0
+
+    See Also
+    --------
+    ClassifierChain : A multi-label model that arranges binary classifiers
+        into a chain.
+    MultiOutputRegressor : Fits one regressor per target variable.
+
     Examples
     --------
     >>> import numpy as np
     >>> from sklearn.datasets import make_multilabel_classification
     >>> from sklearn.multioutput import MultiOutputClassifier
-    >>> from sklearn.neighbors import KNeighborsClassifier
-
+    >>> from sklearn.linear_model import LogisticRegression
     >>> X, y = make_multilabel_classification(n_classes=3, random_state=0)
-    >>> clf = MultiOutputClassifier(KNeighborsClassifier()).fit(X, y)
+    >>> clf = MultiOutputClassifier(LogisticRegression()).fit(X, y)
     >>> clf.predict(X[-2:])
-    array([[1, 1, 0], [1, 1, 1]])
+    array([[1, 1, 1],
+           [1, 0, 1]])
     """
-    @_deprecate_positional_args
+
     def __init__(self, estimator, *, n_jobs=None):
         super().__init__(estimator, n_jobs=n_jobs)
 
@@ -350,12 +413,15 @@ class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
             The input data.
+
         Y : array-like of shape (n_samples, n_classes)
             The target values.
+
         sample_weight : array-like of shape (n_samples,), default=None
-            Sample weights. If None, then samples are equally weighted.
+            Sample weights. If `None`, then samples are equally weighted.
             Only supported if the underlying classifier supports sample
             weights.
+
         **fit_params : dict of string -> object
             Parameters passed to the ``estimator.fit`` method of each step.
 
@@ -364,15 +430,26 @@ class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
         Returns
         -------
         self : object
+            Returns a fitted instance.
         """
         super().fit(X, Y, sample_weight, **fit_params)
         self.classes_ = [estimator.classes_ for estimator in self.estimators_]
         return self
 
-    @property
-    def predict_proba(self):
-        """Probability estimates.
-        Returns prediction probabilities for each class of each output.
+    def _check_predict_proba(self):
+        if hasattr(self, "estimators_"):
+            # raise an AttributeError if `predict_proba` does not exist for
+            # each estimator
+            [getattr(est, "predict_proba") for est in self.estimators_]
+            return True
+        # raise an AttributeError if `predict_proba` does not exist for the
+        # unfitted estimator
+        getattr(self.estimator, "predict_proba")
+        return True
+
+    @available_if(_check_predict_proba)
+    def predict_proba(self, X):
+        """Return prediction probabilities for each class of each output.
 
         This method will raise a ``ValueError`` if any of the
         estimators do not have ``predict_proba``.
@@ -380,12 +457,12 @@ class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
-            Data
+            The input data.
 
         Returns
         -------
         p : array of shape (n_samples, n_classes), or a list of n_outputs \
-            such arrays if n_outputs > 1.
+                such arrays if n_outputs > 1.
             The class probabilities of the input samples. The order of the
             classes corresponds to that in the attribute :term:`classes_`.
 
@@ -395,54 +472,61 @@ class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
                 ``n_classes``) for that particular output.
         """
         check_is_fitted(self)
-        if not all([hasattr(estimator, "predict_proba")
-                    for estimator in self.estimators_]):
-            raise AttributeError("The base estimator should "
-                                 "implement predict_proba method")
-        return self._predict_proba
-
-    def _predict_proba(self, X):
-        results = [estimator.predict_proba(X) for estimator in
-                   self.estimators_]
+        results = [estimator.predict_proba(X) for estimator in self.estimators_]
         return results
 
     def score(self, X, y):
-        """Returns the mean accuracy on the given test data and labels.
+        """Return the mean accuracy on the given test data and labels.
 
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
-            Test samples
+            Test samples.
 
         y : array-like of shape (n_samples, n_outputs)
-            True values for X
+            True values for X.
 
         Returns
         -------
         scores : float
-            accuracy_score of self.predict(X) versus y
+            Mean accuracy of predicted target versus true target.
         """
         check_is_fitted(self)
         n_outputs_ = len(self.estimators_)
         if y.ndim == 1:
-            raise ValueError("y must have at least two dimensions for "
-                             "multi target classification but has only one")
+            raise ValueError(
+                "y must have at least two dimensions for "
+                "multi target classification but has only one"
+            )
         if y.shape[1] != n_outputs_:
-            raise ValueError("The number of outputs of Y for fit {0} and"
-                             " score {1} should be same".
-                             format(n_outputs_, y.shape[1]))
+            raise ValueError(
+                "The number of outputs of Y for fit {0} and"
+                " score {1} should be same".format(n_outputs_, y.shape[1])
+            )
         y_pred = self.predict(X)
         return np.mean(np.all(y == y_pred, axis=1))
 
     def _more_tags(self):
         # FIXME
-        return {'_skip_test': True}
+        return {"_skip_test": True}
+
+
+def _available_if_base_estimator_has(attr):
+    """Return a function to check if `base_estimator` or `estimators_` has `attr`.
+
+    Helper for Chain implementations.
+    """
+
+    def _check(self):
+        return hasattr(self.base_estimator, attr) or all(
+            hasattr(est, attr) for est in self.estimators_
+        )
+
+    return available_if(_check)
 
 
 class _BaseChain(BaseEstimator, metaclass=ABCMeta):
-    @_deprecate_positional_args
-    def __init__(self, base_estimator, *, order=None, cv=None,
-                 random_state=None):
+    def __init__(self, base_estimator, *, order=None, cv=None, random_state=None):
         self.base_estimator = base_estimator
         self.order = order
         self.cv = cv
@@ -456,8 +540,10 @@ class _BaseChain(BaseEstimator, metaclass=ABCMeta):
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
             The input data.
+
         Y : array-like of shape (n_samples, n_classes)
             The target values.
+
         **fit_params : dict of string -> object
             Parameters passed to the `fit` method of each step.
 
@@ -466,11 +552,11 @@ class _BaseChain(BaseEstimator, metaclass=ABCMeta):
         Returns
         -------
         self : object
+            Returns a fitted instance.
         """
         X, Y = self._validate_data(X, Y, multi_output=True, accept_sparse=True)
 
         random_state = check_random_state(self.random_state)
-        check_array(X, accept_sparse=True)
         self.order_ = self.order
         if isinstance(self.order_, tuple):
             self.order_ = np.array(self.order_)
@@ -478,25 +564,24 @@ class _BaseChain(BaseEstimator, metaclass=ABCMeta):
         if self.order_ is None:
             self.order_ = np.array(range(Y.shape[1]))
         elif isinstance(self.order_, str):
-            if self.order_ == 'random':
+            if self.order_ == "random":
                 self.order_ = random_state.permutation(Y.shape[1])
         elif sorted(self.order_) != list(range(Y.shape[1])):
             raise ValueError("invalid order")
 
-        self.estimators_ = [clone(self.base_estimator)
-                            for _ in range(Y.shape[1])]
+        self.estimators_ = [clone(self.base_estimator) for _ in range(Y.shape[1])]
 
         if self.cv is None:
             Y_pred_chain = Y[:, self.order_]
             if sp.issparse(X):
-                X_aug = sp.hstack((X, Y_pred_chain), format='lil')
+                X_aug = sp.hstack((X, Y_pred_chain), format="lil")
                 X_aug = X_aug.tocsr()
             else:
                 X_aug = np.hstack((X, Y_pred_chain))
 
         elif sp.issparse(X):
             Y_pred_chain = sp.lil_matrix((X.shape[0], Y.shape[1]))
-            X_aug = sp.hstack((X, Y_pred_chain), format='lil')
+            X_aug = sp.hstack((X, Y_pred_chain), format="lil")
 
         else:
             Y_pred_chain = np.zeros((X.shape[0], Y.shape[1]))
@@ -506,13 +591,12 @@ class _BaseChain(BaseEstimator, metaclass=ABCMeta):
 
         for chain_idx, estimator in enumerate(self.estimators_):
             y = Y[:, self.order_[chain_idx]]
-            estimator.fit(X_aug[:, :(X.shape[1] + chain_idx)], y,
-                          **fit_params)
+            estimator.fit(X_aug[:, : (X.shape[1] + chain_idx)], y, **fit_params)
             if self.cv is not None and chain_idx < len(self.estimators_) - 1:
                 col_idx = X.shape[1] + chain_idx
                 cv_result = cross_val_predict(
-                    self.base_estimator, X_aug[:, :col_idx],
-                    y=y, cv=self.cv)
+                    self.base_estimator, X_aug[:, :col_idx], y=y, cv=self.cv
+                )
                 if sp.issparse(X_aug):
                     X_aug[:, col_idx] = np.expand_dims(cv_result, 1)
                 else:
@@ -532,10 +616,9 @@ class _BaseChain(BaseEstimator, metaclass=ABCMeta):
         -------
         Y_pred : array-like of shape (n_samples, n_classes)
             The predicted values.
-
         """
         check_is_fitted(self)
-        X = check_array(X, accept_sparse=True)
+        X = self._validate_data(X, accept_sparse=True, reset=False)
         Y_pred_chain = np.zeros((X.shape[0], len(self.estimators_)))
         for chain_idx, estimator in enumerate(self.estimators_):
             previous_predictions = Y_pred_chain[:, :chain_idx]
@@ -572,7 +655,7 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
         The base estimator from which the classifier chain is built.
 
     order : array-like of shape (n_outputs,) or 'random', default=None
-        If None, the order will be determined by the order of columns in
+        If `None`, the order will be determined by the order of columns in
         the label matrix Y.::
 
             order = [0, 1, 2, ..., Y.shape[1] - 1]
@@ -586,7 +669,7 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
         column 1 in the Y matrix, the second model will make predictions
         for column 3, etc.
 
-        If order is 'random' a random ordering will be used.
+        If order is `random` a random ordering will be used.
 
     cv : int, cross-validation generator or an iterable, default=None
         Determines whether to use cross validated predictions or true
@@ -619,6 +702,29 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
     order_ : list
         The order of labels in the classifier chain.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`. Only defined if the
+        underlying `base_estimator` exposes such an attribute when fit.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
+    See Also
+    --------
+    RegressorChain : Equivalent for regression.
+    MultioutputClassifier : Classifies each output independently rather than
+        chaining.
+
+    References
+    ----------
+    Jesse Read, Bernhard Pfahringer, Geoff Holmes, Eibe Frank, "Classifier
+    Chains for Multi-label Classification", 2009.
+
     Examples
     --------
     >>> from sklearn.datasets import make_multilabel_classification
@@ -641,17 +747,6 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
     array([[0.8387..., 0.9431..., 0.4576...],
            [0.8878..., 0.3684..., 0.2640...],
            [0.0321..., 0.9935..., 0.0625...]])
-
-    See Also
-    --------
-    RegressorChain : Equivalent for regression.
-    MultioutputClassifier : Classifies each output independently rather than
-        chaining.
-
-    References
-    ----------
-    Jesse Read, Bernhard Pfahringer, Geoff Holmes, Eibe Frank, "Classifier
-    Chains for Multi-label Classification", 2009.
     """
 
     def fit(self, X, Y):
@@ -661,32 +756,36 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
             The input data.
+
         Y : array-like of shape (n_samples, n_classes)
             The target values.
 
         Returns
         -------
         self : object
+            Class instance.
         """
         super().fit(X, Y)
-        self.classes_ = [estimator.classes_
-                         for chain_idx, estimator
-                         in enumerate(self.estimators_)]
+        self.classes_ = [
+            estimator.classes_ for chain_idx, estimator in enumerate(self.estimators_)
+        ]
         return self
 
-    @if_delegate_has_method('base_estimator')
+    @_available_if_base_estimator_has("predict_proba")
     def predict_proba(self, X):
         """Predict probability estimates.
 
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
+            The input data.
 
         Returns
         -------
         Y_prob : array-like of shape (n_samples, n_classes)
+            The predicted probabilities.
         """
-        X = check_array(X, accept_sparse=True)
+        X = self._validate_data(X, accept_sparse=True, reset=False)
         Y_prob_chain = np.zeros((X.shape[0], len(self.estimators_)))
         Y_pred_chain = np.zeros((X.shape[0], len(self.estimators_)))
         for chain_idx, estimator in enumerate(self.estimators_):
@@ -703,13 +802,14 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
 
         return Y_prob
 
-    @if_delegate_has_method('base_estimator')
+    @_available_if_base_estimator_has("decision_function")
     def decision_function(self, X):
         """Evaluate the decision_function of the models in the chain.
 
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
+            The input data.
 
         Returns
         -------
@@ -717,6 +817,7 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
             Returns the decision function of the sample for each model
             in the chain.
         """
+        X = self._validate_data(X, accept_sparse=True, reset=False)
         Y_decision_chain = np.zeros((X.shape[0], len(self.estimators_)))
         Y_pred_chain = np.zeros((X.shape[0], len(self.estimators_)))
         for chain_idx, estimator in enumerate(self.estimators_):
@@ -735,8 +836,7 @@ class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
         return Y_decision
 
     def _more_tags(self):
-        return {'_skip_test': True,
-                'multioutput_only': True}
+        return {"_skip_test": True, "multioutput_only": True}
 
 
 class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
@@ -756,7 +856,7 @@ class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
         The base estimator from which the classifier chain is built.
 
     order : array-like of shape (n_outputs,) or 'random', default=None
-        If None, the order will be determined by the order of columns in
+        If `None`, the order will be determined by the order of columns in
         the label matrix Y.::
 
             order = [0, 1, 2, ..., Y.shape[1] - 1]
@@ -799,6 +899,24 @@ class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
     order_ : list
         The order of labels in the classifier chain.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`. Only defined if the
+        underlying `base_estimator` exposes such an attribute when fit.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
+    See Also
+    --------
+    ClassifierChain : Equivalent for classification.
+    MultiOutputRegressor : Learns each output independently rather than
+        chaining.
+
     Examples
     --------
     >>> from sklearn.multioutput import RegressorChain
@@ -810,13 +928,6 @@ class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
     array([[0., 2.],
            [1., 1.],
            [2., 0.]])
-
-    See Also
-    --------
-    ClassifierChain : Equivalent for classification.
-    MultioutputRegressor : Learns each output independently rather than
-        chaining.
-
     """
 
     def fit(self, X, Y, **fit_params):
@@ -826,6 +937,7 @@ class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
             The input data.
+
         Y : array-like of shape (n_samples, n_classes)
             The target values.
 
@@ -838,9 +950,10 @@ class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
         Returns
         -------
         self : object
+            Returns a fitted instance.
         """
         super().fit(X, Y, **fit_params)
         return self
 
     def _more_tags(self):
-        return {'multioutput_only': True}
+        return {"multioutput_only": True}
