@@ -2,19 +2,20 @@
 #          Joris Van den Bossche <jorisvandenbossche@gmail.com>
 # License: BSD 3 clause
 
+import numbers
 import warnings
+
 import numpy as np
 from scipy import sparse
-import numbers
 
-from ..base import BaseEstimator, TransformerMixin
+from ..base import BaseEstimator, TransformerMixin, _OneToOneFeatureMixin
 from ..utils import check_array, is_scalar_nan
 from ..utils.deprecation import deprecated
 from ..utils.validation import check_is_fitted
 from ..utils.validation import _check_feature_names_in
 from ..utils._mask import _get_mask
 
-from ..utils._encode import _encode, _check_unknown, _unique
+from ..utils._encode import _encode, _check_unknown, _unique, _get_counts
 
 
 __all__ = ["OneHotEncoder", "OrdinalEncoder"]
@@ -71,7 +72,9 @@ class _BaseEncoder(TransformerMixin, BaseEstimator):
         # numpy arrays, sparse arrays
         return X[:, feature_idx]
 
-    def _fit(self, X, handle_unknown="error", force_all_finite=True):
+    def _fit(
+        self, X, handle_unknown="error", force_all_finite=True, return_counts=False
+    ):
         self._check_n_features(X, reset=True)
         self._check_feature_names(X, reset=True)
         X_list, n_samples, n_features = self._check_X(
@@ -87,11 +90,18 @@ class _BaseEncoder(TransformerMixin, BaseEstimator):
                 )
 
         self.categories_ = []
+        category_counts = []
 
         for i in range(n_features):
             Xi = X_list[i]
+
             if self.categories == "auto":
-                cats = _unique(Xi)
+                result = _unique(Xi, return_counts=return_counts)
+                if return_counts:
+                    cats, counts = result
+                    category_counts.append(counts)
+                else:
+                    cats = result
             else:
                 cats = np.array(self.categories[i], dtype=Xi.dtype)
                 if Xi.dtype.kind not in "OUS":
@@ -114,8 +124,16 @@ class _BaseEncoder(TransformerMixin, BaseEstimator):
                             " during fit".format(diff, i)
                         )
                         raise ValueError(msg)
+                if return_counts:
+                    category_counts.append(_get_counts(Xi, cats))
+
             self.categories_.append(cats)
 
+        output = {"n_samples": n_samples}
+        if return_counts:
+            output["category_counts"] = category_counts
+        return output
+
     def _transform(
         self, X, handle_unknown="error", force_all_finite=True, warn_on_unknown=False
     ):
@@ -223,7 +241,7 @@ class OneHotEncoder(_BaseEncoder):
         Specifies a methodology to use to drop one of the categories per
         feature. This is useful in situations where perfectly collinear
         features cause problems, such as when feeding the resulting data
-        into a neural network or an unregularized regression.
+        into an unregularized linear regression model.
 
         However, dropping one category breaks the symmetry of the original
         representation and can therefore induce a bias in downstream models,
@@ -244,19 +262,62 @@ class OneHotEncoder(_BaseEncoder):
         .. versionchanged:: 0.23
            The option `drop='if_binary'` was added in 0.23.
 
+        .. versionchanged:: 1.1
+            Support for dropping infrequent categories.
+
     sparse : bool, default=True
         Will return sparse matrix if set True else will return an array.
 
     dtype : number type, default=float
         Desired dtype of output.
 
-    handle_unknown : {'error', 'ignore'}, default='error'
-        Whether to raise an error or ignore if an unknown categorical feature
-        is present during transform (default is to raise). When this parameter
-        is set to 'ignore' and an unknown category is encountered during
-        transform, the resulting one-hot encoded columns for this feature
-        will be all zeros. In the inverse transform, an unknown category
-        will be denoted as None.
+    handle_unknown : {'error', 'ignore', 'infrequent_if_exist'}, \
+                     default='error'
+        Specifies the way unknown categories are handled during :meth:`transform`.
+
+        - 'error' : Raise an error if an unknown category is present during transform.
+        - 'ignore' : When an unknown category is encountered during
+          transform, the resulting one-hot encoded columns for this feature
+          will be all zeros. In the inverse transform, an unknown category
+          will be denoted as None.
+        - 'infrequent_if_exist' : When an unknown category is encountered
+          during transform, the resulting one-hot encoded columns for this
+          feature will map to the infrequent category if it exists. The
+          infrequent category will be mapped to the last position in the
+          encoding. During inverse transform, an unknown category will be
+          mapped to the category denoted `'infrequent'` if it exists. If the
+          `'infrequent'` category does not exist, then :meth:`transform` and
+          :meth:`inverse_transform` will handle an unknown category as with
+          `handle_unknown='ignore'`. Infrequent categories exist based on
+          `min_frequency` and `max_categories`. Read more in the
+          :ref:`User Guide <one_hot_encoder_infrequent_categories>`.
+
+        .. versionchanged:: 1.1
+            `'infrequent_if_exist'` was added to automatically handle unknown
+            categories and infrequent categories.
+
+    min_frequency : int or float, default=None
+        Specifies the minimum frequency below which a category will be
+        considered infrequent.
+
+        - If `int`, categories with a smaller cardinality will be considered
+          infrequent.
+
+        - If `float`, categories with a smaller cardinality than
+          `min_frequency * n_samples`  will be considered infrequent.
+
+        .. versionadded:: 1.1
+            Read more in the :ref:`User Guide <one_hot_encoder_infrequent_categories>`.
+
+    max_categories : int, default=None
+        Specifies an upper limit to the number of output features for each input
+        feature when considering infrequent categories. If there are infrequent
+        categories, `max_categories` includes the category representing the
+        infrequent categories along with the frequent categories. If `None`,
+        there is no limit to the number of output features.
+
+        .. versionadded:: 1.1
+            Read more in the :ref:`User Guide <one_hot_encoder_infrequent_categories>`.
 
     Attributes
     ----------
@@ -275,9 +336,23 @@ class OneHotEncoder(_BaseEncoder):
         - ``drop_idx_ = None`` if all the transformed features will be
           retained.
 
+        If infrequent categories are enabled by setting `min_frequency` or
+        `max_categories` to a non-default value and `drop_idx[i]` corresponds
+        to a infrequent category, then the entire infrequent category is
+        dropped.
+
         .. versionchanged:: 0.23
            Added the possibility to contain `None` values.
 
+    infrequent_categories_ : list of ndarray
+        Defined only if infrequent categories are enabled by setting
+        `min_frequency` or `max_categories` to a non-default value.
+        `infrequent_categories_[i]` are the infrequent categories for feature
+        `i`. If the feature `i` has no infrequent categories
+        `infrequent_categories_[i]` is None.
+
+        .. versionadded:: 1.1
+
     n_features_in_ : int
         Number of features seen during :term:`fit`.
 
@@ -342,6 +417,17 @@ class OneHotEncoder(_BaseEncoder):
     >>> drop_binary_enc.transform([['Female', 1], ['Male', 2]]).toarray()
     array([[0., 1., 0., 0.],
            [1., 0., 1., 0.]])
+
+    Infrequent categories are enabled by setting `max_categories` or `min_frequency`.
+
+    >>> import numpy as np
+    >>> X = np.array([["a"] * 5 + ["b"] * 20 + ["c"] * 10 + ["d"] * 3], dtype=object).T
+    >>> ohe = OneHotEncoder(max_categories=3, sparse=False).fit(X)
+    >>> ohe.infrequent_categories_
+    [array(['a', 'd'], dtype=object)]
+    >>> ohe.transform([["a"], ["b"]])
+    array([[0., 0., 1.],
+           [1., 0., 0.]])
     """
 
     def __init__(
@@ -352,31 +438,113 @@ class OneHotEncoder(_BaseEncoder):
         sparse=True,
         dtype=np.float64,
         handle_unknown="error",
+        min_frequency=None,
+        max_categories=None,
     ):
         self.categories = categories
         self.sparse = sparse
         self.dtype = dtype
         self.handle_unknown = handle_unknown
         self.drop = drop
+        self.min_frequency = min_frequency
+        self.max_categories = max_categories
+
+    @property
+    def infrequent_categories_(self):
+        """Infrequent categories for each feature."""
+        # raises an AttributeError if `_infrequent_indices` is not defined
+        infrequent_indices = self._infrequent_indices
+        return [
+            None if indices is None else category[indices]
+            for category, indices in zip(self.categories_, infrequent_indices)
+        ]
 
     def _validate_keywords(self):
-        if self.handle_unknown not in ("error", "ignore"):
+
+        if self.handle_unknown not in {"error", "ignore", "infrequent_if_exist"}:
             msg = (
-                "handle_unknown should be either 'error' or 'ignore', got {0}.".format(
-                    self.handle_unknown
-                )
+                "handle_unknown should be one of 'error', 'ignore', "
+                f"'infrequent_if_exist' got {self.handle_unknown}."
             )
             raise ValueError(msg)
 
+        if self.max_categories is not None and self.max_categories < 1:
+            raise ValueError("max_categories must be greater than 1")
+
+        if isinstance(self.min_frequency, numbers.Integral):
+            if not self.min_frequency >= 1:
+                raise ValueError(
+                    "min_frequency must be an integer at least "
+                    "1 or a float in (0.0, 1.0); got the "
+                    f"integer {self.min_frequency}"
+                )
+        elif isinstance(self.min_frequency, numbers.Real):
+            if not (0.0 < self.min_frequency < 1.0):
+                raise ValueError(
+                    "min_frequency must be an integer at least "
+                    "1 or a float in (0.0, 1.0); got the "
+                    f"float {self.min_frequency}"
+                )
+
+        self._infrequent_enabled = (
+            self.max_categories is not None and self.max_categories >= 1
+        ) or self.min_frequency is not None
+
+    def _map_drop_idx_to_infrequent(self, feature_idx, drop_idx):
+        """Convert `drop_idx` into the index for infrequent categories.
+
+        If there are no infrequent categories, then `drop_idx` is
+        returned. This method is called in `_compute_drop_idx` when the `drop`
+        parameter is an array-like.
+        """
+        if not self._infrequent_enabled:
+            return drop_idx
+
+        default_to_infrequent = self._default_to_infrequent_mappings[feature_idx]
+        if default_to_infrequent is None:
+            return drop_idx
+
+        # Raise error when explicitly dropping a category that is infrequent
+        infrequent_indices = self._infrequent_indices[feature_idx]
+        if infrequent_indices is not None and drop_idx in infrequent_indices:
+            categories = self.categories_[feature_idx]
+            raise ValueError(
+                f"Unable to drop category {categories[drop_idx]!r} from feature"
+                f" {feature_idx} because it is infrequent"
+            )
+        return default_to_infrequent[drop_idx]
+
     def _compute_drop_idx(self):
+        """Compute the drop indices associated with `self.categories_`.
+
+        If `self.drop` is:
+        - `None`, returns `None`.
+        - `'first'`, returns all zeros to drop the first category.
+        - `'if_binary'`, returns zero if the category is binary and `None`
+          otherwise.
+        - array-like, returns the indices of the categories that match the
+          categories in `self.drop`. If the dropped category is an infrequent
+          category, then the index for the infrequent category is used. This
+          means that the entire infrequent category is dropped.
+        """
         if self.drop is None:
             return None
         elif isinstance(self.drop, str):
             if self.drop == "first":
                 return np.zeros(len(self.categories_), dtype=object)
             elif self.drop == "if_binary":
+                n_features_out_no_drop = [len(cat) for cat in self.categories_]
+                if self._infrequent_enabled:
+                    for i, infreq_idx in enumerate(self._infrequent_indices):
+                        if infreq_idx is None:
+                            continue
+                        n_features_out_no_drop[i] -= infreq_idx.size - 1
+
                 return np.array(
-                    [0 if len(cats) == 2 else None for cats in self.categories_],
+                    [
+                        0 if n_features_out == 2 else None
+                        for n_features_out in n_features_out_no_drop
+                    ],
                     dtype=object,
                 )
             else:
@@ -404,24 +572,28 @@ class OneHotEncoder(_BaseEncoder):
                 raise ValueError(msg.format(len(self.categories_), droplen))
             missing_drops = []
             drop_indices = []
-            for col_idx, (val, cat_list) in enumerate(
+            for feature_idx, (drop_val, cat_list) in enumerate(
                 zip(drop_array, self.categories_)
             ):
-                if not is_scalar_nan(val):
-                    drop_idx = np.where(cat_list == val)[0]
+                if not is_scalar_nan(drop_val):
+                    drop_idx = np.where(cat_list == drop_val)[0]
                     if drop_idx.size:  # found drop idx
-                        drop_indices.append(drop_idx[0])
+                        drop_indices.append(
+                            self._map_drop_idx_to_infrequent(feature_idx, drop_idx[0])
+                        )
                     else:
-                        missing_drops.append((col_idx, val))
+                        missing_drops.append((feature_idx, drop_val))
                     continue
 
-                # val is nan, find nan in categories manually
+                # drop_val is nan, find nan in categories manually
                 for cat_idx, cat in enumerate(cat_list):
                     if is_scalar_nan(cat):
-                        drop_indices.append(cat_idx)
+                        drop_indices.append(
+                            self._map_drop_idx_to_infrequent(feature_idx, cat_idx)
+                        )
                         break
                 else:  # loop did not break thus drop is missing
-                    missing_drops.append((col_idx, val))
+                    missing_drops.append((feature_idx, drop_val))
 
             if any(missing_drops):
                 msg = (
@@ -439,6 +611,191 @@ class OneHotEncoder(_BaseEncoder):
                 raise ValueError(msg)
             return np.array(drop_indices, dtype=object)
 
+    def _identify_infrequent(self, category_count, n_samples, col_idx):
+        """Compute the infrequent indices.
+
+        Parameters
+        ----------
+        category_count : ndarray of shape (n_cardinality,)
+            Category counts.
+
+        n_samples : int
+            Number of samples.
+
+        col_idx : int
+            Index of the current category. Only used for the error message.
+
+        Returns
+        -------
+        output : ndarray of shape (n_infrequent_categories,) or None
+            If there are infrequent categories, indices of infrequent
+            categories. Otherwise None.
+        """
+        if isinstance(self.min_frequency, numbers.Integral):
+            infrequent_mask = category_count < self.min_frequency
+        elif isinstance(self.min_frequency, numbers.Real):
+            min_frequency_abs = n_samples * self.min_frequency
+            infrequent_mask = category_count < min_frequency_abs
+        else:
+            infrequent_mask = np.zeros(category_count.shape[0], dtype=bool)
+
+        n_current_features = category_count.size - infrequent_mask.sum() + 1
+        if self.max_categories is not None and self.max_categories < n_current_features:
+            # stable sort to preserve original count order
+            smallest_levels = np.argsort(category_count, kind="mergesort")[
+                : -self.max_categories + 1
+            ]
+            infrequent_mask[smallest_levels] = True
+
+        output = np.flatnonzero(infrequent_mask)
+        return output if output.size > 0 else None
+
+    def _fit_infrequent_category_mapping(self, n_samples, category_counts):
+        """Fit infrequent categories.
+
+        Defines the private attribute: `_default_to_infrequent_mappings`. For
+        feature `i`, `_default_to_infrequent_mappings[i]` defines the mapping
+        from the integer encoding returned by `super().transform()` into
+        infrequent categories. If `_default_to_infrequent_mappings[i]` is None,
+        there were no infrequent categories in the training set.
+
+        For example if categories 0, 2 and 4 were frequent, while categories
+        1, 3, 5 were infrequent for feature 7, then these categories are mapped
+        to a single output:
+        `_default_to_infrequent_mappings[7] = array([0, 3, 1, 3, 2, 3])`
+
+        Defines private attrite: `_infrequent_indices`. `_infrequent_indices[i]`
+        is an array of indices such that
+        `categories_[i][_infrequent_indices[i]]` are all the infrequent category
+        labels. If the feature `i` has no infrequent categories
+        `_infrequent_indices[i]` is None.
+
+        .. versionadded:: 1.1
+
+        Parameters
+        ----------
+        n_samples : int
+            Number of samples in training set.
+        category_counts: list of ndarray
+            `category_counts[i]` is the category counts corresponding to
+            `self.categories_[i]`.
+        """
+        self._infrequent_indices = [
+            self._identify_infrequent(category_count, n_samples, col_idx)
+            for col_idx, category_count in enumerate(category_counts)
+        ]
+
+        # compute mapping from default mapping to infrequent mapping
+        self._default_to_infrequent_mappings = []
+
+        for cats, infreq_idx in zip(self.categories_, self._infrequent_indices):
+            # no infrequent categories
+            if infreq_idx is None:
+                self._default_to_infrequent_mappings.append(None)
+                continue
+
+            n_cats = len(cats)
+            # infrequent indices exist
+            mapping = np.empty(n_cats, dtype=np.int64)
+            n_infrequent_cats = infreq_idx.size
+
+            # infrequent categories are mapped to the last element.
+            n_frequent_cats = n_cats - n_infrequent_cats
+            mapping[infreq_idx] = n_frequent_cats
+
+            frequent_indices = np.setdiff1d(np.arange(n_cats), infreq_idx)
+            mapping[frequent_indices] = np.arange(n_frequent_cats)
+
+            self._default_to_infrequent_mappings.append(mapping)
+
+    def _map_infrequent_categories(self, X_int, X_mask):
+        """Map infrequent categories to integer representing the infrequent category.
+
+        This modifies X_int in-place. Values that were invalid based on `X_mask`
+        are mapped to the infrequent category if there was an infrequent
+        category for that feature.
+
+        Parameters
+        ----------
+        X_int: ndarray of shape (n_samples, n_features)
+            Integer encoded categories.
+
+        X_mask: ndarray of shape (n_samples, n_features)
+            Bool mask for valid values in `X_int`.
+        """
+        if not self._infrequent_enabled:
+            return
+
+        for col_idx in range(X_int.shape[1]):
+            infrequent_idx = self._infrequent_indices[col_idx]
+            if infrequent_idx is None:
+                continue
+
+            X_int[~X_mask[:, col_idx], col_idx] = infrequent_idx[0]
+            if self.handle_unknown == "infrequent_if_exist":
+                # All the unknown values are now mapped to the
+                # infrequent_idx[0], which makes the unknown values valid
+                # This is needed in `transform` when the encoding is formed
+                # using `X_mask`.
+                X_mask[:, col_idx] = True
+
+        # Remaps encoding in `X_int` where the infrequent categories are
+        # grouped together.
+        for i, mapping in enumerate(self._default_to_infrequent_mappings):
+            if mapping is None:
+                continue
+            X_int[:, i] = np.take(mapping, X_int[:, i])
+
+    def _compute_transformed_categories(self, i, remove_dropped=True):
+        """Compute the transformed categories used for column `i`.
+
+        1. If there are infrequent categories, the category is named
+        'infrequent_sklearn'.
+        2. Dropped columns are removed when remove_dropped=True.
+        """
+        cats = self.categories_[i]
+
+        if self._infrequent_enabled:
+            infreq_map = self._default_to_infrequent_mappings[i]
+            if infreq_map is not None:
+                frequent_mask = infreq_map < infreq_map.max()
+                infrequent_cat = "infrequent_sklearn"
+                # infrequent category is always at the end
+                cats = np.concatenate(
+                    (cats[frequent_mask], np.array([infrequent_cat], dtype=object))
+                )
+
+        if remove_dropped:
+            cats = self._remove_dropped_categories(cats, i)
+        return cats
+
+    def _remove_dropped_categories(self, categories, i):
+        """Remove dropped categories."""
+        if self.drop_idx_ is not None and self.drop_idx_[i] is not None:
+            return np.delete(categories, self.drop_idx_[i])
+        return categories
+
+    def _compute_n_features_outs(self):
+        """Compute the n_features_out for each input feature."""
+        output = [len(cats) for cats in self.categories_]
+
+        if self.drop_idx_ is not None:
+            for i, drop_idx in enumerate(self.drop_idx_):
+                if drop_idx is not None:
+                    output[i] -= 1
+
+        if not self._infrequent_enabled:
+            return output
+
+        # infrequent is enabled, the number of features out are reduced
+        # because the infrequent categories are grouped together
+        for i, infreq_idx in enumerate(self._infrequent_indices):
+            if infreq_idx is None:
+                continue
+            output[i] -= infreq_idx.size - 1
+
+        return output
+
     def fit(self, X, y=None):
         """
         Fit OneHotEncoder to X.
@@ -458,8 +815,18 @@ class OneHotEncoder(_BaseEncoder):
             Fitted encoder.
         """
         self._validate_keywords()
-        self._fit(X, handle_unknown=self.handle_unknown, force_all_finite="allow-nan")
+        fit_results = self._fit(
+            X,
+            handle_unknown=self.handle_unknown,
+            force_all_finite="allow-nan",
+            return_counts=self._infrequent_enabled,
+        )
+        if self._infrequent_enabled:
+            self._fit_infrequent_category_mapping(
+                fit_results["n_samples"], fit_results["category_counts"]
+            )
         self.drop_idx_ = self._compute_drop_idx()
+        self._n_features_outs = self._compute_n_features_outs()
         return self
 
     def fit_transform(self, X, y=None):
@@ -491,6 +858,9 @@ class OneHotEncoder(_BaseEncoder):
         """
         Transform X using one-hot encoding.
 
+        If there are infrequent categories for a feature, the infrequent
+        categories will be grouped into a single category.
+
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
@@ -505,13 +875,17 @@ class OneHotEncoder(_BaseEncoder):
         """
         check_is_fitted(self)
         # validation of X happens in _check_X called by _transform
-        warn_on_unknown = self.handle_unknown == "ignore" and self.drop is not None
+        warn_on_unknown = self.drop is not None and self.handle_unknown in {
+            "ignore",
+            "infrequent_if_exist",
+        }
         X_int, X_mask = self._transform(
             X,
             handle_unknown=self.handle_unknown,
             force_all_finite="allow-nan",
             warn_on_unknown=warn_on_unknown,
         )
+        self._map_infrequent_categories(X_int, X_mask)
 
         n_samples, n_features = X_int.shape
 
@@ -520,26 +894,18 @@ class OneHotEncoder(_BaseEncoder):
             # We remove all the dropped categories from mask, and decrement all
             # categories that occur after them to avoid an empty column.
             keep_cells = X_int != to_drop
-            n_values = []
             for i, cats in enumerate(self.categories_):
-                n_cats = len(cats)
-
                 # drop='if_binary' but feature isn't binary
                 if to_drop[i] is None:
                     # set to cardinality to not drop from X_int
-                    to_drop[i] = n_cats
-                    n_values.append(n_cats)
-                else:  # dropped
-                    n_values.append(n_cats - 1)
+                    to_drop[i] = len(cats)
 
             to_drop = to_drop.reshape(1, -1)
             X_int[X_int > to_drop] -= 1
             X_mask &= keep_cells
-        else:
-            n_values = [len(cats) for cats in self.categories_]
 
         mask = X_mask.ravel()
-        feature_indices = np.cumsum([0] + n_values)
+        feature_indices = np.cumsum([0] + self._n_features_outs)
         indices = (X_int + feature_indices[:-1]).ravel()[mask]
 
         indptr = np.empty(n_samples + 1, dtype=int)
@@ -567,6 +933,9 @@ class OneHotEncoder(_BaseEncoder):
         feature with the unknown category has a dropped category, the dropped
         category will be its inverse.
 
+        For a given input feature, if there is an infrequent category,
+        'infrequent_sklearn' will be used to represent the infrequent category.
+
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape \
@@ -583,34 +952,38 @@ class OneHotEncoder(_BaseEncoder):
 
         n_samples, _ = X.shape
         n_features = len(self.categories_)
-        if self.drop_idx_ is None:
-            n_transformed_features = sum(len(cats) for cats in self.categories_)
-        else:
-            n_transformed_features = sum(
-                len(cats) - 1 if to_drop is not None else len(cats)
-                for cats, to_drop in zip(self.categories_, self.drop_idx_)
-            )
+
+        n_features_out = np.sum(self._n_features_outs)
 
         # validate shape of passed X
         msg = (
             "Shape of the passed X data is not correct. Expected {0} columns, got {1}."
         )
-        if X.shape[1] != n_transformed_features:
-            raise ValueError(msg.format(n_transformed_features, X.shape[1]))
+        if X.shape[1] != n_features_out:
+            raise ValueError(msg.format(n_features_out, X.shape[1]))
+
+        transformed_features = [
+            self._compute_transformed_categories(i, remove_dropped=False)
+            for i, _ in enumerate(self.categories_)
+        ]
 
         # create resulting array of appropriate dtype
-        dt = np.find_common_type([cat.dtype for cat in self.categories_], [])
+        dt = np.find_common_type([cat.dtype for cat in transformed_features], [])
         X_tr = np.empty((n_samples, n_features), dtype=dt)
 
         j = 0
         found_unknown = {}
 
+        if self._infrequent_enabled:
+            infrequent_indices = self._infrequent_indices
+        else:
+            infrequent_indices = [None] * n_features
+
         for i in range(n_features):
-            if self.drop_idx_ is None or self.drop_idx_[i] is None:
-                cats = self.categories_[i]
-            else:
-                cats = np.delete(self.categories_[i], self.drop_idx_[i])
-            n_categories = len(cats)
+            cats_wo_dropped = self._remove_dropped_categories(
+                transformed_features[i], i
+            )
+            n_categories = cats_wo_dropped.shape[0]
 
             # Only happens if there was a column with a unique
             # category. In this case we just fill the column with this
@@ -622,8 +995,12 @@ class OneHotEncoder(_BaseEncoder):
             sub = X[:, j : j + n_categories]
             # for sparse X argmax returns 2D matrix, ensure 1D array
             labels = np.asarray(sub.argmax(axis=1)).flatten()
-            X_tr[:, i] = cats[labels]
-            if self.handle_unknown == "ignore":
+            X_tr[:, i] = cats_wo_dropped[labels]
+
+            if self.handle_unknown == "ignore" or (
+                self.handle_unknown == "infrequent_if_exist"
+                and infrequent_indices[i] is None
+            ):
                 unknown = np.asarray(sub.sum(axis=1) == 0).flatten()
                 # ignored unknown categories: we have a row of all zero
                 if unknown.any():
@@ -645,7 +1022,8 @@ class OneHotEncoder(_BaseEncoder):
                         )
                     # we can safely assume that all of the nulls in each column
                     # are the dropped value
-                    X_tr[dropped, i] = self.categories_[i][self.drop_idx_[i]]
+                    drop_idx = self.drop_idx_[i]
+                    X_tr[dropped, i] = transformed_features[i][drop_idx]
 
             j += n_categories
 
@@ -667,6 +1045,9 @@ class OneHotEncoder(_BaseEncoder):
     def get_feature_names(self, input_features=None):
         """Return feature names for output features.
 
+        For a given input feature, if there is an infrequent category, the most
+        'infrequent_sklearn' will be used as a feature name.
+
         Parameters
         ----------
         input_features : list of str of shape (n_features,)
@@ -679,22 +1060,21 @@ class OneHotEncoder(_BaseEncoder):
             Array of feature names.
         """
         check_is_fitted(self)
-        cats = self.categories_
+        cats = [
+            self._compute_transformed_categories(i)
+            for i, _ in enumerate(self.categories_)
+        ]
         if input_features is None:
             input_features = ["x%d" % i for i in range(len(cats))]
-        elif len(input_features) != len(self.categories_):
+        elif len(input_features) != len(cats):
             raise ValueError(
                 "input_features should have length equal to number of "
-                "features ({}), got {}".format(
-                    len(self.categories_), len(input_features)
-                )
+                "features ({}), got {}".format(len(cats), len(input_features))
             )
 
         feature_names = []
         for i in range(len(cats)):
             names = [input_features[i] + "_" + str(t) for t in cats[i]]
-            if self.drop_idx_ is not None and self.drop_idx_[i] is not None:
-                names.pop(self.drop_idx_[i])
             feature_names.extend(names)
 
         return np.array(feature_names, dtype=object)
@@ -709,7 +1089,8 @@ class OneHotEncoder(_BaseEncoder):
 
             - If `input_features` is `None`, then `feature_names_in_` is
               used as feature names in. If `feature_names_in_` is not defined,
-              then names are generated: `[x0, x1, ..., x(n_features_in_)]`.
+              then the following input feature names are generated:
+              `["x0", "x1", ..., "x(n_features_in_ - 1)"]`.
             - If `input_features` is an array-like, then `input_features` must
               match `feature_names_in_` if `feature_names_in_` is defined.
 
@@ -719,19 +1100,21 @@ class OneHotEncoder(_BaseEncoder):
             Transformed feature names.
         """
         check_is_fitted(self)
-        cats = self.categories_
         input_features = _check_feature_names_in(self, input_features)
+        cats = [
+            self._compute_transformed_categories(i)
+            for i, _ in enumerate(self.categories_)
+        ]
 
         feature_names = []
         for i in range(len(cats)):
             names = [input_features[i] + "_" + str(t) for t in cats[i]]
-            if self.drop_idx_ is not None and self.drop_idx_[i] is not None:
-                names.pop(self.drop_idx_[i])
             feature_names.extend(names)
-        return np.asarray(feature_names, dtype=object)
+
+        return np.array(feature_names, dtype=object)
 
 
-class OrdinalEncoder(_BaseEncoder):
+class OrdinalEncoder(_OneToOneFeatureMixin, _BaseEncoder):
     """
     Encode categorical features as an integer array.
 
@@ -777,6 +1160,12 @@ class OrdinalEncoder(_BaseEncoder):
 
         .. versionadded:: 0.24
 
+    encoded_missing_value : int or np.nan, default=np.nan
+        Encoded value of missing categories. If set to `np.nan`, then the `dtype`
+        parameter must be a float dtype.
+
+        .. versionadded:: 1.1
+
     Attributes
     ----------
     categories_ : list of arrays
@@ -820,6 +1209,23 @@ class OrdinalEncoder(_BaseEncoder):
     >>> enc.inverse_transform([[1, 0], [0, 1]])
     array([['Male', 1],
            ['Female', 2]], dtype=object)
+
+    By default, :class:`OrdinalEncoder` is lenient towards missing values by
+    propagating them.
+
+    >>> import numpy as np
+    >>> X = [['Male', 1], ['Female', 3], ['Female', np.nan]]
+    >>> enc.fit_transform(X)
+    array([[ 1.,  0.],
+           [ 0.,  1.],
+           [ 0., nan]])
+
+    You can use the parameter `encoded_missing_value` to encode missing values.
+
+    >>> enc.set_params(encoded_missing_value=-1).fit_transform(X)
+    array([[ 1.,  0.],
+           [ 0.,  1.],
+           [ 0., -1.]])
     """
 
     def __init__(
@@ -829,11 +1235,13 @@ class OrdinalEncoder(_BaseEncoder):
         dtype=np.float64,
         handle_unknown="error",
         unknown_value=None,
+        encoded_missing_value=np.nan,
     ):
         self.categories = categories
         self.dtype = dtype
         self.handle_unknown = handle_unknown
         self.unknown_value = unknown_value
+        self.encoded_missing_value = encoded_missing_value
 
     def fit(self, X, y=None):
         """
@@ -903,13 +1311,38 @@ class OrdinalEncoder(_BaseEncoder):
                     self._missing_indices[cat_idx] = i
                     continue
 
-        if np.dtype(self.dtype).kind != "f" and self._missing_indices:
-            raise ValueError(
-                "There are missing values in features "
-                f"{list(self._missing_indices)}. For OrdinalEncoder to "
-                "passthrough missing values, the dtype parameter must be a "
-                "float"
-            )
+        if self._missing_indices:
+            if np.dtype(self.dtype).kind != "f" and is_scalar_nan(
+                self.encoded_missing_value
+            ):
+                raise ValueError(
+                    "There are missing values in features "
+                    f"{list(self._missing_indices)}. For OrdinalEncoder to "
+                    f"encode missing values with dtype: {self.dtype}, set "
+                    "encoded_missing_value to a non-nan value, or "
+                    "set dtype to a float"
+                )
+
+            if not is_scalar_nan(self.encoded_missing_value):
+                # Features are invalid when they contain a missing category
+                # and encoded_missing_value was already used to encode a
+                # known category
+                invalid_features = [
+                    cat_idx
+                    for cat_idx, categories_for_idx in enumerate(self.categories_)
+                    if cat_idx in self._missing_indices
+                    and 0 <= self.encoded_missing_value < len(categories_for_idx)
+                ]
+
+                if invalid_features:
+                    # Use feature names if they are avaliable
+                    if hasattr(self, "feature_names_in_"):
+                        invalid_features = self.feature_names_in_[invalid_features]
+                    raise ValueError(
+                        f"encoded_missing_value ({self.encoded_missing_value}) "
+                        "is already used to encode a known category in features: "
+                        f"{invalid_features}"
+                    )
 
         return self
 
@@ -934,7 +1367,7 @@ class OrdinalEncoder(_BaseEncoder):
 
         for cat_idx, missing_idx in self._missing_indices.items():
             X_missing_mask = X_int[:, cat_idx] == missing_idx
-            X_trans[X_missing_mask, cat_idx] = np.nan
+            X_trans[X_missing_mask, cat_idx] = self.encoded_missing_value
 
         # create separate category for unknown values
         if self.handle_unknown == "use_encoded_value":
@@ -979,7 +1412,7 @@ class OrdinalEncoder(_BaseEncoder):
 
             # replace values of X[:, i] that were nan with actual indices
             if i in self._missing_indices:
-                X_i_mask = _get_mask(X[:, i], np.nan)
+                X_i_mask = _get_mask(X[:, i], self.encoded_missing_value)
                 labels[X_i_mask] = self._missing_indices[i]
 
             if self.handle_unknown == "use_encoded_value":
